const AsteriskConn = require('./AsteriskConn');
const Fila = require('xcontact-lib/models/Fila');
const Agente = require('xcontact-lib/models/Agente');
const AgenteAmi = require('./models/AgenteAmi');
const AgenteFilas = require('xcontact-lib/models/AgenteFilas');
const LigacoesAmi = require('./models/LigacoesAmi');
const log = require("node-live-log")("Ami.js");
const EventEmitter = require('events');
class MyEmitter extends EventEmitter { }
const emitter = new MyEmitter();
const moment = require('moment')
const _ = require('underscore');

module.exports = class Ami {
    constructor() {
        this.filas = [];
        this.transferencias = [];
        this.StatusQueue = [];
        this.AddQueue = [];
        this.RemoveQueue = [];
        this.transferidas = [];
    }

    ProcessaQueues () {
        const vm = this;
        setInterval(() => {
            if (this.StatusQueue.length > 0) {
                const allElements = this.StatusQueue.splice(0, this.StatusQueue.length)
                const arrayUniqueByKey = [...new Map(allElements.map(item =>
                    [item.id, item])).values()]
                arrayUniqueByKey.forEach(async (event) => {
                    emitter.emit('AgenteStatus', event)
                    // this.processaEventoUnico(event)
                })
            }
        }, 1000)
    }

    on (event, fn) {
        return emitter.addListener(event, fn);
    }

    /**
     * Limpa eventos do Asterisk
     * @param {Object} response 
     */
    limpa (response) {
        const { lines, EOL, variables, privilege, event, ...ev } = response;
        return ev;
    }

    resetFila (fila) {
        fila.ligacoes_atendidas = [];
        fila.ligacoes_ativas = [];
        fila.ligacoes_abandonadas = [];
        fila.ligacoes_abandonadas_efetivas = [];
        fila.ligacoes_aguardando = [];
        fila.ligacoes_efetuadas = [];
        fila.ligacoes_transbordadas = [];

        fila.total_transbordo = 0;
        fila.total_espera = 0;
        fila.esperas_maior = 0;
        fila.esperas_menor = 0;
        fila.ligacoes_dentro_meta = 0;
        fila.abandonos_maior = 0;
        fila.abandonos_menor = 0;
        fila.total_falando = 0;
        fila.total_abandono = 0;
        fila.max_espera = 0;
        fila.max_falando = 0;
        fila.max_abandono = 0;
        fila.ligacoes_efetuadas_totais = 0;

        // Suporte ao omni
        fila.MsgEnviadas = 0
        fila.MsgRecebidas = 0
        fila.TotalAtendimentosEntrantes = 0
        fila.TotalAtendimentosSaintes = 0
        fila.TotalAtendimentosEncerrados = 0
        fila.TotalAtendimentosTransbordados = 0
        fila.obj_atendimentos = []
        fila.idEntrante = []
        fila.tempo_total_resposta = 0
        fila.total_mensagens_resposta = 0
        fila.tempo_primeira_resposta = 0
        fila.total_primeira_resposta = 0
        fila.tmr = 0
        fila.tmpr = 0
        fila.tmaxr = 0
        fila.tmaxpr = 0
        fila.AtendimentosAguardando = []
        fila.clientes_atendidos = []
        fila.AtendimentosEmAndamento = []

        if (!fila.agentes) fila.agentes = [];
    }

    resetAgente (agente) {
        agente.active_calls = [];
        agente.atendimento_ativo = [];
        agente.ligacoes_atendidas = 0;
        agente.ligacoes_efetuadas = 0;
        agente.ligacoes_efetuadas_totais = 0;
        agente.ligacoes_recebidas = 0;
        agente.ligacoes_naoatendidas = 0;
        agente.ligacoes_naocompletadas = 0;
        agente.atendimentos_recebidos = 0;
        agente.atendimentos_efetuados = 0;
        agente.total_falando = 0;
        agente.total_espera = 0;
        agente.tempo_primeira_resposta = 0;
        agente.total_primeira_resposta = 0;
        agente.tempo_total_resposta = 0;
        agente.total_mensagens_resposta = 0;
        agente.tempo_total_resposta = 0;
        agente.max_espera = 0;
        agente.max_falando = 0;
        agente.tmaxr = 0;
        agente.tmaxpr = 0;
        agente.statusDate = moment().unix();
        agente.status_date = new Date().getTime();
        agente.time = 0;
    }

    reset () {
        this.filas.forEach(fila => {
            this.resetFila(fila);
            fila.agentes.forEach(agente => {
                this.resetAgente(agente);
            })
        });
        this.transferencias = [];
    }

    _calcFilaStatus (fila) {
        return {
            nome: fila.name,
            agentes: fila.agentes.length,
            aguardando: fila.ligacoes_aguardando.length,
            atendidas: fila.ligacoes_atendidas.length,
            efetuadas: fila.ligacoes_efetuadas.length,
            abandonadas: fila.ligacoes_abandonadas.length,
            transbordadas: fila.ligacoes_transbordadas.length,
            transbordos: fila.total_transbordo,
            tme: Math.round(fila.total_espera / fila.ligacoes_atendidas.length) || 0,
            tma: Math.round(fila.total_falando / (fila.ligacoes_atendidas.length + fila.ligacoes_efetuadas.length)) || 0,
            tmab: Math.round(fila.total_abandono / fila.ligacoes_abandonadas.length) || 0,
            tmr: Math.round(fila.tempo_total_resposta / fila.total_mensagens_resposta) || 0,
            tmpr: Math.round(fila.tempo_primeira_resposta / fila.total_primeira_resposta) || 0,
            tmaxr: fila.tmaxr,
            tmaxpr: fila.tmaxpr,
            max_espera: fila.max_espera,
            max_falando: fila.max_falando,
            max_abandono: fila.max_abandono
        }
    }

    getAllFilasStatus () {
        return this.filas.map(fila => {
            return this._calcFilaStatus(fila);
        })
    }

    getFilaStatus (NomeFila) {
        const fila = this.filas.find(f => f.name == NomeFila);
        if (fila) {
            return this._calcFilaStatus(fila);
        }
    }

    async addAgente (NomeFila, agente) {
        if (this.filas.length <= 0) return;
        if (!agente.name) agente.name = agente.membername;
        if (!agente.ramal) agente.ramal = agente.stateinterface;
        log.info(`[addAgente] Adicionando ${agente.name} na fila ${NomeFila}`);

        let fila = this.filas.find(f => f.name === NomeFila);
        if (fila) {
            if (!fila.agentes) fila.agentes = [];
            // Pesquisa por ramal pois o comando de logoff remove por ramal
            const agFila = fila.agentes.find(ag => ag.ramal === agente.ramal)
            if (agFila) {
                agFila.status = 1;
                agFila.status_date = new Date().getTime();
                agFila.statusDate = moment().unix();
                agFila.pausa = parseInt(agente.paused);
                agFila.pausa_motivo = agente.pausedreason;

                emitter.emit('AgenteAdicionado', agFila);

                return agFila;
            } else {
                // Se o agente não está na fila, busca no banco de dados
                log.info(`[addAgente] Agente ${agente.name} não encontrado na fila ${NomeFila}, buscando no banco...`);

                const AgDb = await Agente.query.require(false)
                    .first()
                    .where({ fullname: agente.name })
                    .fetch();

                if (AgDb) {
                    // Busca a fila padrão (primeira fila do agente no banco)
                    const AgDbFila = await AgenteFilas.query.require(false)
                    .where({ id_agente: AgDb.id })
                    .orderBy('id')
                    .first()
                    .fetch()
                    
                    agente.id = AgDb.id;
                    agente.modo = AgDb.modo;
                    agente.fila_padrao = AgDbFila ? AgDbFila.fila : undefined;
                    agente.max_chat_atende = AgDb.max_chat_atende;
                    agente.edita_rota = AgDb.edita_rota;
                    agente.visualiza_todos_clientes = AgDb.visualiza_todos_clientes;

                    // Cria uma nova instância do agente
                    const agenteAmi = {
                        id: agente.id,
                        nome: agente.name,
                        status: 1, // Define o status ativo
                        ramal: agente.ramal || AgDb.ramal,
                        fila: NomeFila,
                        fila_padrao: agente.fila_padrao,
                        pausa: parseInt(agente.paused),
                        pausa_motivo: agente.pausedreason,
                        active_calls: [],
                        atendimento_ativo: [],
                        max_chat_atende: agente.max_chat_atende,
                        status_date: new Date().getTime(),
                        statusDate: moment().unix(),
                        ligacoes_atendidas: 0,
                        ligacoes_efetuadas: 0,
                        ligacoes_efetuadas_totais: 0,
                        ligacoes_recebidas: 0,
                        ligacoes_naoatendidas: 0,
                        ligacoes_naocompletadas: 0,
                        atendimentos_recebidos: 0,
                        atendimentos_efetuados: 0,
                        total_falando: 0,
                        total_espera: 0,
                        tempo_primeira_resposta: 0,
                        total_primeira_resposta: 0,
                        tempo_total_resposta: 0,
                        total_mensagens_resposta: 0,
                        max_espera: 0,
                        max_falando: 0,
                        tmaxr: 0,
                        tmaxpr: 0,
                    };
                    // Adiciona o agente à fila
                    fila.agentes.push(agenteAmi);

                    // Emite um evento indicando que o agente foi adicionado
                    emitter.emit('AgenteAdicionado', agenteAmi);

                    return agenteAmi;
                } else {
                log.warn(`[addAgente] Agente ${agente.name} não encontrado no banco de dados.`);
                return null;
                }
            }
            
            // const AgDb = await Agente.query.require(false).first()
            //     .where({ fullname: agente.name })
            //     .fetch();
            // if (AgDb) {
            //     // Busca a fila padrão, primeira fila do agente.
            //     const AgDbFila = await AgenteFilas.query.require(false).first()
            //         .orderBy('id')
            //         .where({ id_agente: AgDb.id })
            //         .fetch();
            //     agente.id = AgDb.id;
            //     agente.modo = AgDb.modo;
            //     agente.fila_padrao = AgDbFila.fila || undefined;
            //     agente.max_chat_atende = AgDb.max_chat_atende;
            //     agente.edita_rota = AgDb.edita_rota;
            //     agente.visualiza_todos_clientes = AgDb.visualiza_todos_clientes;
            // } else return
            // const agenteAmi = new AgenteAmi({
            //     id: agente.id,
            //     nome: agente.name,
            //     status: agente.status ? agente.status : 1,
            //     ramal: agente.ramal || AgDb.ramal,
            //     fila: fila.name,
            //     fila_padrao: agente.fila_padrao,
            //     pausa: parseInt(agente.paused),
            //     pausa_motivo: agente.pausedreason,
            //     active_calls: [],
            //     atendimento_ativo: [],
            //     max_chat_atende: agente.max_chat_atende,
            //     status_date: new Date().getTime(),
            //     statusDate: moment().unix(),
            //     ligacoes_atendidas: 0,
            //     ligacoes_efetuadas: 0,
            //     ligacoes_efetuadas_totais: 0,
            //     ligacoes_recebidas: 0,
            //     ligacoes_naoatendidas: 0,
            //     ligacoes_naocompletadas: 0,
            //     atendimentos_recebidos: 0,
            //     atendimentos_efetuados: 0,
            //     total_falando: 0,
            //     total_espera: 0,
            //     tempo_primeira_resposta: 0,
            //     total_primeira_resposta: 0,
            //     tempo_total_resposta: 0,
            //     total_mensagens_resposta: 0,
            //     max_espera: 0,
            //     max_falando: 0,
            //     tmaxr: 0,
            //     tmaxpr: 0,
            // });
            // if (!fila.agentes) fila.agentes = [];
            // fila.agentes.push(agenteAmi);

            // emitter.emit('AgenteAdicionado', agenteAmi);

            // return agente;
        } else {
            log.warn(`[addAgente] Fila ${NomeFila} não encontrada na lista.`);
            return null;
        }
    }

    _delAgente (NomeFila, agente) {
        if (this.filas.length <= 0) return;
        if (!agente.name) agente.name = agente.membername;
        if (!agente.ramal) agente.ramal = agente.stateinterface;
        log.info(`[delAgente] Removendo ${agente.name} da fila ${NomeFila}`);

        let fila = this.filas.find(f => f.name === NomeFila);
        if (fila) {
            if (fila.agentes.length <= 0) return;

            const index = fila.agentes.findIndex(ag => ag.ramal === agente.ramal);
            if (index !== -1) {
                const agenteAmi = fila.agentes[index];
                emitter.emit('AgenteRemovido', agenteAmi);
                agenteAmi.status = 12;
                agenteAmi.status_date = new Date().getTime();
                agenteAmi.statusDate = moment().unix();
                // fila.agentes.splice(index, 1);
            }
        }
        else {
            log.warn(`[delAgente] Fila ${NomeFila} não encontrada na lista.`);
        }
    }

    getAgenteByRamal (ramal) {
        // Nunca use forEach, forEach não interromp com return e sempre dá problema em async
        for (let i = 0; i < this.filas.length; i++) {
            if (this.filas[i].agentes.length > 0) {
                const agente = this.filas[i].agentes.find(agente => agente.ramal === ramal);
                if (agente) {
                    return agente;
                }
            }
        }
    }

    getAgentePorNome (nome) {
        // Nunca use forEach, forEach não interromp com return e sempre dá problema em async
        for (let i = 0; i < this.filas.length; i++) {
            if (this.filas[i].agentes.length > 0) {
                const agente = this.filas[i].agentes.find(agente => agente.nome === nome);
                if (agente) {
                    return agente;
                }
            }
        }
    }

    async syncFila (request) {
        try {
            let filadb = await Fila.fetchByName(request.fila);
            let filaarray = this.filas.find(f => f.name == request.fila);
            if (filaarray) {
                filaarray = Object.assign(filaarray, filadb);
                filaarray.nome = filaarray.name;
                filaarray.tipo = filaarray.queue_type;
                filaarray.fila = filaarray.name;
                emitter.emit('FilaAtualizada', filaarray);
                return filaarray;
            }
            else {
                this.resetFila(filadb);
                this.filas.push(filadb);
                filadb.nome = filadb.name;
                filadb.tipo = filadb.queue_type;
                filadb.fila = filadb.name;
                emitter.emit('FilaAtualizada', filadb);
                return filadb;
            }
        } catch (error) {
            log.error(`Erro ao carregar a fila ${request.fila}`, error);
        }
    }

    async RemoverFila (request) {
        const index = this.filas.findIndex(f => f.name == request.fila);
        if (index !== -1) {
            await Fila.query
                .require(false)
                .where({ name: request.fila })
                .delete();

            let filaremovida = this.filas.splice(index, 1);
            filaremovida[0].fila = filaremovida[0].name;

            emitter.emit('FilaRemovida', filaremovida[0]);
            return filaremovida[0];
        }
        else {
            return { error: true, msg: 'Fila não encontrada' };
        }
    }

    async AdicionarFilaAgente (request) {
        log.debug('[AdicionarFilaAgente] Request: ', request);
        try {
            const agf = new AgenteFilas({ id_agente: parseInt(request.id_agente), fila: request.fila });
            let ret = await agf.save();

            emitter.emit('FilaAdicionadaAgente', { id_agente: parseInt(request.id_agente), fila: request.fila });

            return ret;
        } catch (error) {
            log.error('[AdicionarFilaAgente] Erro: ', error);
            return null;
        }
    }

    async RemoverFilaAgente (request) {
        log.debug('[RemoverFilaAgente] Request: ', request);
        try {
            let ret = await AgenteFilas.query
                .require(false)
                .where({ id_agente: parseInt(request.id_agente) })
                .where({ fila: request.fila })
                .delete();

            emitter.emit('FilaRemovidaAgente', { id_agente: parseInt(request.id_agente), fila: request.fila });

            return ret;
        } catch (error) {
            log.error('[RemoverFilaAgente] Erro: ', error);
            return null;
        }
    }

    adionarLigacaoNaFila (ev) {
        log.debug('[LigacaoEntrouNafila]', ev)
        const fila = this.filas.find(f => f.name == ev.queue);
        if (fila) {
            const ligacaoExistente = fila.ligacoes_aguardando.find(lig => lig.callid === ev.linkedid)
            if (ligacaoExistente) return
            const ligacao = new LigacoesAmi({
                callid: ev.linkedid,
                numero: ev.calleridnum,
                nome: ev.calleridname,
                fila: ev.queue,
                posicao_inicial: parseInt(ev.position),
                statusDate: moment().unix(),
                total: parseInt(ev.count),
                inicio: new Date().getTime(),
                sentido: LigacoesAmi.SentidoMap.get('Entrante'),
            })
            fila.ligacoes_aguardando.push(ligacao);

            const agenteLigando = this.getAgenteByRamal((`SIP/${ev.calleridnum}`));
            if (agenteLigando && agenteLigando.fila_padrao === fila.name) {
                ligacao.mesmaFila = true
                
                const ligEfetIndex = fila.ligacoes_efetuadas.findIndex((lig) => lig.callid === ev.linkedid)
                if (ligEfetIndex !== -1) {
                    fila.ligacoes_efetuadas.splice(ligEfetIndex, 1)
                }

                const ligIndex = fila.ligacoes_ativas.findIndex((lig) => lig.callid === ev.linkedid && lig.agente_nome === agenteLigando.nome)
                if (ligIndex !== -1) {
                    fila.ligacoes_ativas.splice(ligIndex, 1)
                }
                fila.ligacoes_efetuadas_totais--
            }
            if (ligacao.numero) {
                fila.clientes_atendidos.push(ligacao.numero)
            }
            fila.clientes_atendidos = _.uniq(fila.clientes_atendidos)

            emitter.emit('FilaLigacaoAdicionada', ligacao);
        }
        else {
            log.error(`[LigacaoEntrouNaFila] Fila ${ev.queue} não encontrada`)
        }
    }

    abandonarLigacaoNaFila (ev) {
        log.debug('[LigacaoAbandonouNaFila]', ev)
        const fila = this.filas.find(f => f.name == ev.queue);
        if (fila) {
            const index = fila.ligacoes_aguardando.findIndex(ag => ag.callid == ev.linkedid);
            if (index !== -1) {
                const ligacao = fila.ligacoes_aguardando[index];
                fila.ligacoes_aguardando.splice(index, 1);

                ligacao.fim = new Date().getTime();
                ligacao.posicao_final = parseInt(ev.position);

                var diff = Math.abs(ligacao.fim - ligacao.inicio);
                ligacao.espera = Math.round(diff / 1000);

                // Abandono
                // if (ev.reason && ev.reason == 'NORMAL') {
                fila.total_abandono += ligacao.espera;
                if (ligacao.espera > fila.max_abandono) fila.max_abandono = ligacao.espera;
                if (ligacao.espera > 60) {
                    fila.abandonos_maior++
                }
                else {
                    fila.abandonos_menor++
                }

                fila.ligacoes_abandonadas.push(ligacao);

                const abandonoEfetivo = fila.ligacoes_abandonadas_efetivas.find(
                    (call) => call.numero === ligacao.numero)
                if (!abandonoEfetivo) {
                    fila.ligacoes_abandonadas_efetivas.push(ligacao)
                }
                emitter.emit('FilaLigacaoAbandonou', ligacao);
                // }
                // Transbordo
                // else {
                //     fila.ligacoes_transbordadas.push(ligacao);
                //     fila.total_transbordo += ligacao.espera;
                //     emitter.emit('FilaLigacaoTransbordou', ligacao);
                // }
            }
            else {
                log.warn(`[LigacaoAbandonouNaFila] Não achei a ligação de callid ${ligacao.callid} na fila ${fila.name}`);
            }
        }
        else {
            log.error(`[LigacaoAbandonouNaFila] Fila ${ev.queue} não encontrada`)
        }
    }

    atenderLiacaoNaFila (ev) {
        log.debug('[AgenteAtendeu]', ev)
        const fila = this.filas.find(f => f.name == ev.queue);
        if (fila) {
            const index = fila.ligacoes_aguardando.findIndex(ag => ag.callid == ev.linkedid);
            if (index !== -1) {
                const ligacao = fila.ligacoes_aguardando[index];
                const agente = fila.agentes.find(agente => agente.nome === ev.membername);
                fila.ligacoes_aguardando.splice(index, 1);
                ligacao.atendida = new Date().getTime();

                var diff = Math.abs(ligacao.atendida - ligacao.inicio);
                ligacao.espera = Math.round(diff / 1000);
                fila.total_espera += ligacao.espera;

                if (ligacao.espera > 60) {
                    fila.esperas_maior++
                }
                else {
                    fila.esperas_menor++
                }

                if (ligacao.espera <= fila.nivel_tempo) {
                    ligacao.dentro_meta = true
                    fila.ligacoes_dentro_meta++
                }

                if (!fila.ligacoes_ativas.find(lig => lig.callid === ev.linkedid)) {
                    fila.ligacoes_ativas.push(ligacao);
                }
                fila.ligacoes_atendidas.push(ligacao);
                if (ligacao.espera > fila.max_espera) fila.max_espera = ligacao.espera;

                ligacao.agente_nome = ev.membername
                ligacao.agente_ramal = ev.interface
                ligacao.ring_agente_segundos = ev.ringtime

                if (fila.ligacoes_abandonadas_efetivas) {
                    const indexesAbandonadas = fila.ligacoes_abandonadas_efetivas.reduce((acc, call, i) => {
                        if (call.numero === ligacao.numero) {
                            acc.push(i)
                        }
                        return acc
                    }, [])
                    indexesAbandonadas.reverse().forEach((indexAb) => {
                        fila.ligacoes_abandonadas_efetivas.splice(indexAb, 1)
                    })
                } else {
                    return
                }

                if (agente) {
                    agente.total_espera += ligacao.espera;
                    agente.ligacoes_atendidas++;
                    if (ligacao.espera > agente.max_espera) agente.max_espera = ligacao.espera;
                    if (agente.fila_padrao !== ev.queue) {
                        const agentePadrao = this.getAgenteByRamal(ev.interface.split('-')[0])
                        let ligacaoPadrao = agentePadrao.active_calls.find(lig => lig.callid === ev.linkedid);
                        if (ligacaoPadrao) {
                            ligacaoPadrao.fila = ev.queue
                        }
                    }
                }

                emitter.emit('FilaLigacaoAtendida', ligacao);
            }
            else {
                log.warn(`[LigacaoSaiuDaFila] Não achei a ligação de callid ${ev.linkedid} na fila ${fila.name}`);
            }
        }
    }

    desligarLigacaoNaFila (ev) {
        log.debug('[AgenteDesligou]', ev)
        const fila = this.filas.find(f => f.name == ev.queue);
        if (fila) {
            const index = fila.ligacoes_atendidas.findIndex(ag => ag.callid == ev.linkedid);
            if (index !== -1) {
                const ligacao = fila.ligacoes_atendidas[index];
                const indexAtiva = fila.ligacoes_ativas.findIndex(ag => ag.callid == ev.linkedid);
                const agente = fila.agentes.find(agente => agente.nome === ev.membername);
                if (indexAtiva !== -1) {
                    if (ev.reason === 'transfer' && agente) {
                        const ligIndex = fila.ligacoes_ativas.findIndex((lig) => lig.callid === ev.linkedid && lig.agente_nome === agente.nome)
                        if (ligIndex !== -1) {
                            fila.ligacoes_ativas.splice(ligIndex, 1)
                        }
                    } else {
                        const ligIndex = fila.ligacoes_ativas.findIndex((lig) => lig.callid === ev.linkedid)
                        if (ligIndex !== -1) {
                            fila.ligacoes_ativas.splice(ligIndex, 1)
                        }
                        ligacao.agente_nome = ev.membername
                        ligacao.agente_ramal = ev.interface
                    }
                }

                ligacao.falando = parseInt(ev.talktime);
                if (agente) {
                    const ligIndex = agente.active_calls.findIndex((lig) => lig.callid === ev.linkedid)
                    if (ligIndex !== -1) {
                        agente.active_calls.splice(ligIndex, 1)
                    }

                    agente.total_falando += ligacao.falando;
                    if (ligacao.falando > agente.max_falando) agente.max_falando = ligacao.falando;
                }

                ligacao.motivo_fim = LigacoesAmi.MotivoFimMap.get(ev.reason);
                ligacao.fim = new Date().getTime();

                fila.total_falando += ligacao.falando;

                if (ligacao.falando > fila.max_falando) fila.max_falando = ligacao.falando;
                ligacao.reason = ev.reason
                emitter.emit('FilaLigacaoTerminou', ligacao);
                ligacao.reason = null
            }
        }
    }

    async start () {
        log.info('[AMI] Start: Serviço iniciado');
        // Primeiro vamos carregar as filas no banco pq o Asterisk as vezes não traz todas as filas.
        try {
            const filas = await Fila.query
            .leftJoin(
                AgenteFilas.query.leftJoin(
                    Agente.query.leftJoin(AgenteFilas),
                )
            )
            .fetch();
            this.filas = filas.map(fila => {
                const agf = []
                fila.agenteFilas.map(ag => {
                    if (agf.findIndex(a => a.id === ag.id) === -1) {
                        agf.push(ag);
                    }            
                })
                fila.agentes = []
                agf.map(ag => { 
                    ag.agente.map(a => {
                        if (fila.agentes.findIndex(agente => a.id === agente.id) === -1) {
                            const copy = JSON.parse(JSON.stringify(a));
                            copy.nome = a.fullname;
                            copy.status = 12
                            copy.fila = fila.name
                            copy.fila_padrao = a.agenteFilas.sort((a, b) => a.id - b.id)[0].fila
                            copy.pausa = 0
                            copy.pausa_motivo = ''
                            copy.active_calls = []
                            copy.atendimento_ativo = []
                            copy.status_date = new Date().getTime()
                            copy.statusDate = moment().unix()
                            copy.ligacoes_atendidas = 0
                            copy.ligacoes_efetuadas = 0
                            copy.ligacoes_efetuadas_totais = 0
                            copy.ligacoes_recebidas = 0
                            copy.ligacoes_recebidas = 0
                            copy.ligacoes_naoatendidas = 0
                            copy.ligacoes_naocompletadas = 0
                            copy.atendimentos_recebidos = 0
                            copy.atendimentos_efetuados = 0
                            copy.total_falando = 0
                            copy.total_espera = 0
                            copy.tempo_primeira_resposta = 0
                            copy.total_primeira_resposta = 0
                            copy.tempo_total_resposta = 0
                            copy.total_mensagens_resposta = 0
                            copy.max_espera = 0
                            copy.max_falando = 0
                            copy.tmaxr = 0
                            copy.tmaxpr = 0

                            delete copy.agenteFilas

                            fila.agentes.push(copy);
                        }
                    });
                } );
                delete fila.agenteFilas;
                return fila;
            });
            log.info('Filas carregadas com sucesso');
            this.reset();
        } catch (error) {
            log.error('Erro ao carregar as filas do banco de dados', error);
            this.filas = [];
        }

        this.ProcessaQueues();

        this.asterConn = new AsteriskConn();
        const vm = this;
        this.asterConn.on('ConexaoAmiInciada', event => {
            vm.asterConn.getFilas(undefined, async (retorno) => {
                // Aqui só no interesa os Agentes que estão logados para popular na fila
                if (retorno.response == 'Success') {
                    retorno.events.map(evento => {
                        if (evento.event == 'QueueMember') {
                            vm.addAgente(evento.queue, vm.limpa(evento));
                        }
                    })
                    emitter.emit('Iniciado', { error: false, msg: 'iniciado com sucesso' });
                }
            })
        });

        this.asterConn.on('AgenteAdicionado', async event => {
            const tem = await this.addAgente(event.queue, event)
            if (!tem) {
                log.debug(`Fila ${event.queue} não encontrada, tentando syncronizar com o banco`)
                await this.syncFila({ fila: event.queue })
                this.addAgente(event.queue, event)
            }
        });

        this.asterConn.on('AgenteRemovido', event => {
            this._delAgente(event.queue, event);
        });

        this.asterConn.on('AgenteStatus', event => {
            log.debug('[AgenteStatus]', event)
            if (this.filas.length <= 0) return;

            if (!event.name) event.name = event.membername;
            log.info(`[AgenteStatus] Agente ${event.name} da fila ${event.queue} mudou para status ${event.status}`);

            let fila = this.filas.find(f => f.name === event.queue);
            if (fila) {
                if (fila.agentes.length <= 0) return;

                const agt = fila.agentes.find(ag => ag.nome === event.name);
                if (agt) {
                    agt.status = parseInt(event.status);
                    agt.status_date = new Date().getTime();
                    agt.statusDate = moment().unix();
                    agt.pausa = parseInt(event.paused);
                    agt.pausa_motivo = event.pausedreason;

                    this.StatusQueue.push(agt)
                    // emitter.emit('AgenteStatus', agt);
                }
            } else {
                log.warn(`[AgenteStatus] Fila ${event.queue} não encontrada na lista.`);
            }
        });

        this.asterConn.on('TransferenciaDireta', event => {
            log.debug('TransferenciaDireta', event)
            this.transferencias.push(event.transfererlinkedid);

            const agente = this.getAgenteByRamal(event.transfererchannel.split('-')[0]);
            const agenteDestino = this.getAgenteByRamal(`SIP/${event.extension}`);
            if (agente) {
                const ligacaoAgente = agente.active_calls.find(lig => lig.callid === event.transfereelinkedid)
                const fila = this.filas.find(f => f.name === ligacaoAgente.fila)
                let ligacao = fila.ligacoes_ativas.find(lig => lig.callid === event.transfereelinkedid)
                if (ligacao && agenteDestino) {
                    const filaPadrao = this.filas.find(f => f.name === agenteDestino.fila_padrao)
                    ligacao.agente_nome = agenteDestino.nome
                    ligacao.agente_ramal = agenteDestino.ramal
                    const ligIndex = fila.ligacoes_ativas.findIndex((lig) => lig.callid === event.transfereelinkedid)
                    if (ligIndex !== -1) {
                        fila.ligacoes_ativas.splice(ligIndex, 1)
                    }
                    filaPadrao.ligacoes_ativas.push(ligacao)
                    if (fila.name !== filaPadrao.name) {
                        ligacao.fila = filaPadrao.name
                        filaPadrao.ligacoes_atendidas.push(ligacao)
                    }
                }

                const evento = {
                    ligacao,
                    fila: ligacaoAgente.fila,
                    agente,
                    agenteDestino
                }
                emitter.emit('TransferenciaDireta', evento)
            } else {
                log.warn('Não havia algum agente')
            }
        });

        this.asterConn.on('TransferenciaConsulta', event => {
            log.debug('TransferenciaConsulta', event)
            this.transferencias.push(event.origtransfererlinkedid);

            const agente = this.getAgenteByRamal(event.origtransfererchannel.split('-')[0]);
            let agenteDestino = null;
            if (event.transfertargetchannel){
                agenteDestino = this.getAgenteByRamal(event.transfertargetchannel.split('-')[0]);
            }
            let segundaFila = null;
            if (agente) {
                let ligacaoAgente = agente.active_calls.find(lig => lig.callid === event.origtransfererlinkedid)
                let fila = this.filas.find(f => f.name === ligacaoAgente.fila)
                let ligacao = fila.ligacoes_ativas.find(lig => lig.callid === event.origtransfererlinkedid)
                if (agenteDestino) {
                    if (!agenteDestino.active_calls) agenteDestino.active_calls = [];
                    let segundaLigacao = agenteDestino.active_calls.find(lig => lig.callid === event.transfertargetlinkedid)
                    segundaFila = this.filas.find(f => f.name === segundaLigacao.fila)
                    if (ligacao) {
                        ligacao.agente_nome = agenteDestino.nome
                        ligacao.agente_ramal = agenteDestino.ramal
                        if (fila.name !== segundaFila.name) {
                            ligacao.fila = segundaFila.name
                            
                            const ligIndex = fila.ligacoes_ativas.findIndex((lig) => lig.callid === event.origtransfererlinkedid)
                            if (ligIndex !== -1) {
                                fila.ligacoes_ativas.splice(ligIndex, 1)
                            }
                            segundaFila.ligacoes_ativas.push(ligacao)
                        } else {
                            segundaFila.ligacoes_efetuadas.push(ligacao)
                            segundaFila.ligacoes_efetuadas_totais++
                        }
                    }
                    agenteDestino.active_calls.push(ligacao);

                    const ligIndex = agenteDestino.active_calls.findIndex((lig) => lig.callid === event.transfertargetlinkedid)
                    if (ligIndex !== -1) {
                        agenteDestino.active_calls.splice(ligIndex, 1)
                    }
                    agenteDestino.ligacoes_recebidas--
                    agenteDestino.ligacoes_efetuadas++
                    agenteDestino.ligacoes_efetuadas_totais++
                } else {
                    for (let i = 0; i < this.filas.length; i++) {
                        segundaFila = this.filas[i];

                        for (let j = 0; j < segundaFila.agentes.length; j++) {
                            let agenteDois = segundaFila.agentes[j];
                            const segundaLigacao = agenteDois.active_calls.findIndex((ag) => ag.callid === event.secondtransfererlinkedid)
                            if (segundaLigacao !== -1) {
                                agenteDois.active_calls.splice(segundaLigacao, 1)
                                agenteDestino = agenteDois
                                break
                            }
                        }
                        

                        const ligacaoAguardando = segundaFila.ligacoes_aguardando.find(lig => lig.callid === event.secondtransfererlinkedid);
                        if (ligacaoAguardando) {
                            this.transferidas.push(event.secondtransfererlinkedid)
                            const secLigIndex = segundaFila.ligacoes_aguardando.findIndex((lig) => lig.callid === event.secondtransfererlinkedid)
                            if (secLigIndex !== -1) {
                                segundaFila.ligacoes_aguardando.splice(secLigIndex, 1)
                            }
                            if (ligacao) {
                                ligacao.agente_nome = ''
                                ligacao.agente_ramal = ''
                                if (fila.name !== segundaFila.name) {
                                    ligacao.fila = segundaFila.name
                                }   
                                const ligIndex = fila.ligacoes_ativas.findIndex((lig) => lig.callid === event.origtransfererlinkedid)
                                if (ligIndex !== -1) {
                                    fila.ligacoes_ativas.splice(ligIndex, 1)
                                }
                                segundaFila.ligacoes_aguardando.push(ligacao)
                            }
                            break;
                        }
                    }
                }
                const evento = {
                    ligacao,
                    fila: fila.name,
                    segundaFila: segundaFila.name || segundaFila.nome,
                    agente,
                    agenteDestino,
                    callidDestino: event.transfertargetlinkedid || event.secondtransfererlinkedid
                }
                emitter.emit('TransferenciaConsulta', evento)
            } else {
                let fila = null
                let ligacao = null
                for (let i = 0; i < this.filas.length; i++) {
                    fila = this.filas[i];
                    ligacao = fila.ligacoes_ativas.find(lig => lig.callid === event.origtransfererlinkedid)
                    if (ligacao) {  
                        const ligIndex = fila.ligacoes_ativas.findIndex((lig) => lig.callid === event.origtransfererlinkedid)
                        if (ligIndex !== -1) {
                            fila.ligacoes_ativas.splice(ligIndex, 1)
                        }
                        if (agenteDestino) {
                            if (!agenteDestino.active_calls) agenteDestino.active_calls = [];
                            let segundaLigacao = agenteDestino.active_calls.find(lig => lig.callid === event.transfertargetlinkedid)
                            segundaFila = this.filas.find(f => f.name === segundaLigacao.fila)
                            
                            ligacao.agente_nome = agenteDestino.nome
                            ligacao.agente_ramal = agenteDestino.ramal
                            if (fila.name !== segundaFila.name) {
                                ligacao.fila = segundaFila.name
                                
                                const ligIndex = fila.ligacoes_ativas.findIndex((lig) => lig.callid === event.origtransfererlinkedid)
                                if (ligIndex !== -1) {
                                    fila.ligacoes_ativas.splice(ligIndex, 1)
                                }
                                segundaFila.ligacoes_ativas.push(ligacao)
                            } else {
                                segundaFila.ligacoes_efetuadas.push(ligacao)
                                segundaFila.ligacoes_efetuadas_totais++
                            }
                            
                            agenteDestino.active_calls.push(ligacao);
    
                            const ligIndex = agenteDestino.active_calls.findIndex((lig) => lig.callid === event.transfertargetlinkedid)
                            if (ligIndex !== -1) {
                                agenteDestino.active_calls.splice(ligIndex, 1)
                            }
                            agenteDestino.ligacoes_recebidas--
                            agenteDestino.ligacoes_efetuadas++
                            agenteDestino.ligacoes_efetuadas_totais++
                        } else {
                            for (let i = 0; i < this.filas.length; i++) {
                                segundaFila = this.filas[i];
                                const ligacaoAguardando = segundaFila.ligacoes_aguardando.find(lig => lig.callid === event.secondtransfererlinkedid);
                                if (ligacaoAguardando) {
                                    const secLigIndex = segundaFila.ligacoes_aguardando.findIndex((lig) => lig.callid === event.secondtransfererlinkedid)
                                    if (secLigIndex !== -1) {
                                        segundaFila.ligacoes_aguardando.splice(secLigIndex, 1)
                                    }
                                    ligacao.agente_nome = ''
                                    ligacao.agente_ramal = ''
                                    if (fila.name !== segundaFila.name) {
                                        ligacao.fila = segundaFila.name
                                    }   
                                    const ligIndex = fila.ligacoes_ativas.findIndex((lig) => lig.callid === event.origtransfererlinkedid)
                                    if (ligIndex !== -1) {
                                        fila.ligacoes_ativas.splice(ligIndex, 1)
                                    }
                                    segundaFila.ligacoes_aguardando.push(ligacao) 
                                }
                            }
                            this.transferidas.push(event.secondtransfererlinkedid)
                        }
                        break;
                    }
                }
                const evento = {
                    ligacao,
                    fila: fila.name,
                    segundaFila: segundaFila.name || segundaFila.nome,
                    agente,
                    agenteDestino,
                    callidDestino: event.transfertargetlinkedid || event.secondtransfererlinkedid
                }
                emitter.emit('TransferenciaConsulta', evento)
            }
        });

        this.asterConn.on('LigacaoStatus', event => {
            log.debug('LigacaoStatus', event);
            const agente = this.getAgenteByRamal(event.channel.split('-')[0]);
            // log.info(agente);
            if (agente) {
                if (!agente.active_calls) agente.active_calls = [];

                let nova = false;
                let ligacao = agente.active_calls.find(lig => lig.callid === event.linkedid);
                if (!ligacao) {
                    nova = true;
                    ligacao = new LigacoesAmi({
                        callid: event.linkedid,
                        inicio: new Date().getTime(),
                        statusDate: moment().unix(),
                        fila: agente.fila_padrao,
                        agente_id: agente.id,
                        agente_nome: agente.nome,
                        agente_ramal: agente.ramal,
                        transferida: false,
                    })
                }

                // Ligacao sainte
                if (event.channelstate == '4' || event.exten.length === 0) {
                    ligacao.status = LigacoesAmi.StatusMap.get("Chamando");
                    ligacao.sentido = LigacoesAmi.SentidoMap.get("Sainte");
                    ligacao.numero = event.exten;
                    agente.ligacoes_efetuadas_totais++;
                    let fila = this.filas.find(f => f.name === agente.fila_padrao);
                    if (fila) {
                        fila.ligacoes_efetuadas_totais++
                    }
                    if (ligacao.numero) {
                        fila.clientes_atendidos.push(ligacao.numero)
                    }
                    fila.clientes_atendidos = _.uniq(fila.clientes_atendidos)
                }
                // Ligacao entrante
                if (event.channelstate == '5' && event.exten.length > 0) {
                    ligacao.status = LigacoesAmi.StatusMap.get("Chamando");
                    ligacao.sentido = LigacoesAmi.SentidoMap.get('Entrante');
                    ligacao.numero = event.connectedlinenum;
                    ligacao.nome = event.connectedlinename;
                }
                if (event.channelstate == '6') {
                    ligacao.atendida = new Date().getTime();
                    ligacao.status = LigacoesAmi.StatusMap.get("Ativa");
                    // Contabiliza ligação só se completou
                    if (ligacao.sentido == LigacoesAmi.SentidoMap.get('Entrante')) {
                        agente.ligacoes_recebidas++;
                    } else {
                        agente.ligacoes_efetuadas++;
                        // Registra a ligação sainte da fila
                        let fila = this.filas.find(f => f.name === agente.fila_padrao);
                        if (fila) {
                            fila.ligacoes_efetuadas.push(ligacao);
                        }
                        if (ligacao.status === 2) {
                            if (!fila.ligacoes_ativas.find(lig => lig.callid === event.linkedid) && !this.transferidas.includes(ligacao.callid)) {
                                fila.ligacoes_ativas.push(ligacao);
                            }
                            if (fila.ligacoes_abandonadas_efetivas) {
                                const indexesAbandonadas = fila.ligacoes_abandonadas_efetivas.reduce((acc, call, i) => {
                                    if (call.numero === ligacao.numero) {
                                        acc.push(i)
                                    }
                                    return acc
                                }, [])
                                indexesAbandonadas.reverse().forEach((indexAb) => {
                                    fila.ligacoes_abandonadas_efetivas.splice(indexAb, 1)
                                })
                            } else {
                                return
                            }
                        }
                    }
                }
                const transferida = this.transferidas.find(callid => callid === event.linkedid)
                if (nova && ligacao && !transferida) {
                    agente.active_calls.push(ligacao);
                }
                if (transferida) {
                    ligacao.transferida = true;
                }
            
                let fila = this.filas.find(f => f.name === agente.fila_padrao);
                if (fila) {
                    let ligacaoFila = fila.ligacoes_efetuadas.find(lig => lig.callid === event.linkedid)
                    if (ligacaoFila) {
                        ligacaoFila.status = ligacao.status
                        ligacaoFila.sentido = ligacao.sentido
                    }
                }

                emitter.emit('AgenteLigacao', ligacao);
            } else {
                log.warn(`Nenhum agente encontrado para ramal ${event.channel.split('-')[0]}`)
            }
        });

        this.asterConn.on('LigacaoFim', event => {
            log.debug('LigacaoFim', event);
            const agente = this.getAgenteByRamal(event.channel.split('-')[0]);
            if (agente) {
                // log.debug('Agente encontrado', agente)
                const ligacao = agente.active_calls.find(lig => lig.callid === event.linkedid);
                if (ligacao) {
                    // Contabiliza ligações não atendidas ou não completadas
                    if (event.channelstate != '6') {
                        if (ligacao.sentido == LigacoesAmi.SentidoMap.get('Entrante')) {
                            agente.ligacoes_naoatendidas++;
                            ligacao.status = LigacoesAmi.StatusMap.get('Nao_Atendida');
                        }
                        else {
                            agente.ligacoes_naocompletadas++;
                            ligacao.status = LigacoesAmi.StatusMap.get('Nao_Completada');
                        }
                    } else {
                        ligacao.status = LigacoesAmi.StatusMap.get('Finalizada');
                    }

                    ligacao.fim = new Date().getTime();

                    //Substituir o array de ligações ativas pelo resultado do filter não é uma boa prática, pois o array de ligações ativas pode ser alterado em outro lugar.
                    //agente.active_calls = agente.active_calls.filter(lig => lig.callid !== event.linkedid);

                    // Esse for é mais seguro, pois ele não altera o array original, ele apenas remove os itens do array.
                    // for (let i = 0; (i = agente.active_calls.findIndex(lig => lig.callid === event.linkedid)) >= 0; agente.active_calls.splice(i, 1));

                    let agIndex = agente.active_calls.findIndex(
                        (lig) =>  lig.callid === event.linkedid
                    );
                      
                    while (agIndex !== -1) {
                        agente.active_calls.splice(agIndex, 1);

                        agIndex = agente.active_calls.findIndex(
                          (lig) =>  lig.callid === event.linkedid
                        );
                    }

                    let fila = this.filas.find(f => f.name === ligacao.fila);
                    if (fila) {
                        const ligIndex = fila.ligacoes_ativas.findIndex((lig) => lig.callid === event.linkedid && lig.agente_nome === agente.nome)
                        if (ligIndex !== -1) {
                            fila.ligacoes_ativas.splice(ligIndex, 1)
                        }

                        ligacao.falando = parseInt((ligacao.fim - ligacao.atendida) / 1000);
                        if (ligacao.sentido === LigacoesAmi.SentidoMap.get('Sainte') && ligacao.falando) {
                            fila.total_falando += ligacao.falando;
                            if (ligacao.falando > fila.max_falando) fila.max_falando = ligacao.falando;

                            const agenteFila = fila.agentes.find(ag => ag.nome === agente.nome)
                            if (agenteFila) {
                                agente.total_falando += ligacao.falando;
                                if (ligacao.falando > agente.max_falando) agente.max_falando = ligacao.falando;
                            }
                        }
                    }
                    const agenteLigando = this.getAgenteByRamal((`SIP/${event.exten}`));
                    if (agenteLigando) {
                        ligacao.agenteLigando = agenteLigando.id
                        // for (let i = 0; (i = agenteLigando.active_calls.findIndex(lig => lig.callid === event.linkedid)) >= 0; agenteLigando.active_calls.splice(i, 1));

                        let agIndex = agenteLigando.active_calls.findIndex(
                            (lig) =>  lig.callid === event.linkedid
                        );
                          
                        while (agIndex !== -1) {
                            agenteLigando.active_calls.splice(agIndex, 1);
    
                            agIndex = agenteLigando.active_calls.findIndex(
                              (lig) =>  lig.callid === event.linkedid
                            );
                        }
                    }
                    ligacao.agente_id = agente.id
                    ligacao.cause = event.cause
                    emitter.emit('AgenteLigacao', ligacao);
                } else {
                    log.warn('Ligação não encontrada para o agente', event)
                    // Vou emitir o evento mesmo assim com a duração zerada para fins de ser removido no front
                    const ligacao = new LigacoesAmi({
                        callid: event.linkedid,
                        inicio: new Date().getTime(),
                        fim: new Date().getTime(),
                        falando: 0,
                        status: LigacoesAmi.StatusMap.get('Finalizada'),
                        fila: agente.fila_padrao,
                        agente_id: agente.id,
                        agente_nome: agente.nome,
                        agente_ramal: agente.ramal
                    })
                    const agenteLigando = this.getAgenteByRamal((`SIP/${event.exten}`));
                    if (agenteLigando) {
                        ligacao.agenteligando = agenteLigando.id
                        // for (let i = 0; (i = agenteLigando.active_calls.findIndex(lig => lig.callid === event.linkedid)) >= 0; agenteLigando.active_calls.splice(i, 1));
                        let agIndex = agenteLigando.active_calls.findIndex(
                            (lig) =>  lig.callid === event.linkedid
                        );
                          
                        while (agIndex !== -1) {
                            agenteLigando.active_calls.splice(agIndex, 1);
    
                            agIndex = agenteLigando.active_calls.findIndex(
                              (lig) =>  lig.callid === event.linkedid
                            );
                        }
                    }
                    
                    for (let i = 0; i < this.filas.length; i++) {
                        let fila = this.filas[i];
                        let ligacao = fila.ligacoes_ativas.find(lig => lig.callid === event.linkedid)
                        if (ligacao) {
                            const ligIndex = fila.ligacoes_ativas.findIndex((lig) => lig.callid === event.linkedid)
                            if (ligIndex !== -1) {
                                fila.ligacoes_ativas.splice(ligIndex, 1)
                            }
                            break
                        }
                    }
                    ligacao.cause = event.cause
                    emitter.emit('AgenteLigacao', ligacao);
                }
            } else {
                log.warn('Agente não encontrado para ligação', event)
                for (let i = 0; i < this.filas.length; i++) {
                    let fila = this.filas[i];
                    let ligacao = fila.ligacoes_ativas.find(lig => lig.callid === event.linkedid)
                    if (ligacao) {
                        const ligIndex = fila.ligacoes_ativas.findIndex((lig) => lig.callid === event.linkedid)
                        if (ligIndex !== -1) {
                            fila.ligacoes_ativas.splice(ligIndex, 1)
                        }
                        ligacao.cause = event.cause
                        emitter.emit('AgenteLigacao', ligacao);
                        break
                    }
                }
            }
        });

        this.asterConn.on('AgentePausa', event => {
            log.debug('[AgentePausa]', event);
            if (!event.name) event.name = event.membername;

            let fila = this.filas.find(f => f.name === event.queue);
            if (fila) {
                if (fila.agentes.length <= 0) return;

                const agt = fila.agentes.find(ag => ag.nome === event.name);
                if (agt) {
                    agt.evento = 'AgentePausa'
                    agt.status = parseInt(event.status);
                    agt.status_date = new Date().getTime();
                    agt.statusDate = moment().unix();
                    agt.pausa = parseInt(event.paused);
                    agt.pausa_motivo = event.pausedreason;

                    this.StatusQueue.push(agt)
                    //emitter.emit('AgenteStatus', agt);
                }
            }
            else {
                log.warn((`[AgentePausa] Fila ${event.queue} não encontrada na lista.`));
            }
        })

        this.asterConn.on('LigacaoEntrouNaFila', ev => this.adionarLigacaoNaFila(ev));

        this.asterConn.on('LigacaoAbandonouNaFila', ev => this.abandonarLigacaoNaFila(ev));

        this.asterConn.on('AgenteAtendeu', ev => this.atenderLiacaoNaFila(ev));

        this.asterConn.on('AgenteDesligou', ev => this.desligarLigacaoNaFila(ev));
    }
    /**
     * Retorna todas as filas
     * @returns Array
     */
    getFilas () {
        return this.filas.map(fila => {
            let copy = Object.assign({}, fila);
            copy.nome = fila.name;
            copy.tipo = fila.queue_type;

            copy.atendidas = fila.ligacoes_atendidas.length;
            copy.efetuadas = fila.ligacoes_efetuadas.length;
            copy.abandonadas = fila.ligacoes_abandonadas.length;
            copy.transbordadas = fila.ligacoes_transbordadas.length;

            copy.tme = Math.round(fila.total_espera / fila.ligacoes_atendidas.length) || 0;
            copy.tma = Math.round(fila.total_falando / (fila.ligacoes_atendidas.length + fila.ligacoes_efetuadas.length)) || 0;
            copy.tmab = Math.round(fila.total_abandono / fila.ligacoes_abandonadas.length) || 0;
            copy.tmr = Math.round(fila.tempo_total_resposta / fila.total_mensagens_resposta) || 0;
            copy.tmpr = Math.round(fila.tempo_primeira_resposta / fila.total_primeira_resposta) || 0;

            copy.ligacoes_aguardando.forEach(lig => {
                if (lig){
                    const diff = moment().diff(lig.inicio, 'miliseconds');
                    lig.time = diff;
                }
            });

            copy.AtendimentosAguardando.forEach(at => {
                if (at){
                    const diff = moment().diff(at.data, 'miliseconds');
                    at.time = diff;
                }
            });

            copy.agentes.forEach(ag => {
                if (ag){
                    const diff = moment().diff(ag.status_date, 'miliseconds');
                    ag.time = diff;
                }
            });
            delete copy.ligacoes_atendidas;
            delete copy.ligacoes_efetuadas;
            delete copy.ligacoes_transbordadas;
            return copy;
        })
    }
}