# Xcontact AMI
Micro serviço que prove comunicação com o Asterisk efetuando ações de agentes, filas e monitorando eventos.

## Project setup
```
yarn install ou npm install
```

## Project start
```
yarn start ou npm run start
```

## Dev Config
Para o ambiente dev use um arqwuivo `.env`

Variáveis, ex:
```
PGHOST=10.11.31.4
AMI_HOST=10.11.31.4
XCLOG_LEVEL=debug
```

## Instalar como serviço - LINUX
```
yarn install-service ou npm run install-service
```

### Login
Para chamar metodos e receber eventos via websocket os usuários precisam efetuar login no socket.

> A API verifica quais filas os usuários tem acesso e apenas publica eventos e permite métodos das filas atribuidas a eles.

* `login_adm` - Usuários adm(Supervisores) devem chamar o método `login_adm` com seu usuário e senha conforme abaixo:

Exemplo:
```js
const client = require("socket.io-client")("https://localhost:3000", {
    "rejectUnauthorized": false
});

client.emit('login_adm', { nome: 'admin', senha: '********' }, resposta => {
    console.log(`Login resposta: ${JSON.stringify(resposta)}`);
    if (resposta.error) {
        console.log(`Verifique os dados de acesso`);
    }
    else {
        // Login ok
        console.log('Dae sim...logado');
    }
});
```

* `login_agente` - Agentes devem chamar o método `login_agente` com seu usuário e senha conforme abaixo:

Exemplo:
```js
const client = require("socket.io-client")("https://localhost:3000", {
    "rejectUnauthorized": false
});

client.emit('login_agente', { nome: 'agente1', senha: '*****' }, resposta => {
    console.log(`Login resposta: ${JSON.stringify(resposta)}`);
    if (resposta.error) {
        console.log(`Verifique os dados de acesso`);
    }
    else {
        // Login ok
        console.log('Dae sim...logado');
    }
});
```

## Eventos
Os eventos são enviados apenas para agentes ou usuários adm logados no socket e após chamar o método `monitorar_filas`. 

> A API verifica quais filas os usuários tem acesso e apenas publica eventos e permite métodos das filas atribuidas a eles.

Exemplo:
```js
const client = require("socket.io-client")("https://localhost:3000", {
    "rejectUnauthorized": false
});

client.on('connect', () => {
    console.log('Parece que conectou');

    client.emit('login_adm', { nome: 'admin', senha: '*******' }, resposta => {
        console.log(`Login resposta: ${JSON.stringify(resposta)}`);
        if (resposta.error) {
            console.log(`Verifique os dados de acesso`);
        }
        else {
            // Login ok
            console.log('Dae sim...logado');

            client.emit('monitorar_filas', {}, resposta => {
                console.log(`monitorar_filas resposta: ${JSON.stringify(resposta)}`);
            });
        }
    });

});
```


### Eventos de filas
Os eventos de filas são eventos que ocorem nas filas, atualização de parametros da fila, logins de agentes nas filas e ligações.
Os eventos de filas são publicados apenas para quem chamar o método `monitorar_filas` e apenas para as filas que o usuário ou agente tem acesso.

* Como receber os eventos das filas: Chame o método `monitorar_filas`

```js
client.emit('monitorar_filas', {}, resposta => {
    console.log(`monitorar_filas resposta: ${JSON.stringify(resposta)}`);
});
```

#### FilaAtualizada
----
Evento emitido quando o cadastro de uma fila é atualizada.

* **RETORNO**

Dados da Fila
```json
{
    "name": "NOME_DA_FILA",
    "id": ID_DA_FILA,
    ...
}
```

#### FilaRemovida
----
Evento emitido quando uma fila é removida do cadastro.

* **RETORNO**

Dados da Fila
```json
{
    "name": "NOME_DA_FILA",
    "id": ID_DA_FILA,
    ...
}
```

#### FilaRemovidaAgente
----
Evento eimitido quando uma fila é removida do cadastro do agente.

* **RETORNO**

```json
{
    "id_agente": ID_DO_AGENTE,
    "fila": "NOME_DA_FILA"
}
```

#### FilaAdicionadaAgente
----
Evento emitido quando uma nova fila é adicionada ao cadastro do agente. Como os eventos são enviados apenas para quem tem acesso a fila esse evento também será enviado para o agente se ele estiver logado.

* **RETORNO**

```json
{
    "id_agente": ID_DO_AGENTE,
    "fila": "NOME_DA_FILA"
}
```

#### FilaLigacaoAdicionada
----
Evento emitido quando uma ligação entra na fila.

* **RETORNO**

```json
{
    "callid": "1619715342.168",
    "numero": "<unknown>",
    "nome": "<unknown>",
    "fila": "Atendimento",
    "posicao_inicial": 1,
    "total": 1,
    "inicio": 1619715340285    
}
```

#### FilaLigacaoAtendida
----
Evento emitido quando uma ligação é atendida.
> OBS: Para saber qual agente atendeu a ligação você deve ouvir o evento `AgenteLigacao`

* **RETORNO**

```json
{
    "callid": "1619715342.168",
    "numero": "<unknown>",
    "nome": "<unknown>",
    "fila": "Atendimento",
    "agente_nome": "Teste",
    "agente_ramal": "SIP/1234",
    "ring_agente_segundos": 25,
    "posicao_inicial": 1,
    "total": 1,
    "inicio": 1619715340285,
    "atendida": 1619715347085,
    "espera": null
}
```

#### FilaLigacaoTerminou
----
Evento emitido quando uma ligação que estava ativa (atendida) termina.

* **RETORNO**

```json
{
  "callid": "1619715342.168",
  "numero": "<unknown>",
  "nome": "<unknown>",
  "fila": "Atendimento",
  "agente_nome": "Teste",
  "agente_ramal": "SIP/1234",    
  "posicao_inicial": 1,
  "total": 1,
  "inicio": 1619715340285,
  "atendida": 1619715347085,
  "espera": null,
  "motivo_fim": 2,
  "fim": 1619715350469,
  "falando": 4
}
```

#### FilaLigacaoAbandonou
----
Evento emitido quando uma ligação abandona.

* **RETORNO**

```json
{
  "callid": "1619719668.171",
  "numero": "<unknown>",
  "nome": "<unknown>",
  "fila": "Atendimento",
  "posicao_inicial": 1,
  "total": 1,
  "inicio": 1619719665739,
  "fim": 1619719678388,
  "posicao_final": 1,
  "espera": 13
}
```

#### FilaLigacaoTransbordou
----
Evento emitido quando uma ligação transborda.

* **RETORNO**

```json
{
  "callid": "1619719668.171",
  "numero": "<unknown>",
  "nome": "<unknown>",
  "fila": "Atendimento",
  "posicao_inicial": 1,
  "total": 1,
  "inicio": 1619719665739,
  "fim": 1619719678388,
  "posicao_final": 1,
  "espera": 13
}
```

#### AgenteAdicionado
----
Evento emitido quando um agente loga em uma fila.

* **RETORNO**

```json
{
  "id": 1,
  "nome": "Teste",
  "status": 1,
  "ramal": "SIP/1005",
  "fila": "Atendimento",
  "fila_padrao": "Atendimento",
  "pausa": 0,
  "pausa_motivo": "",
  "max_chat_atende": 5,
  "status_date": 1619728970870,
  "ligacoes_efetuadas": 0,
  "ligacoes_recebidas": 0,
  "ligacoes_naoatendidas": 0,
  "ligacoes_naocompletadas": 0
}
```

#### AgenteRemovido
----
Evento emitido quando um agente desloga de uma fila.

* **RETORNO**

```json
{
  "id": 1,
  "nome": "Teste",
  "status": 1,
  "ramal": "SIP/1005",
  "fila": "Atendimento",
  "fila_padrao": "Atendimento",
  "pausa": 1,
  "pausa_motivo": "Intervalo",
  "max_chat_atende": 5,
  "status_date": 1619723823684,
  "ligacoes_efetuadas": 1,
  "ligacoes_recebidas": 1,
  "ligacoes_naoatendidas": 0,
  "ligacoes_naocompletadas": 0,
  "ligacoes_ativas": []
}
```

#### AgenteStatus
----
Evento emitido quando um agente muda de status. Ex: Entra em pausa, saí da pausa...

- Status
  - 0 - AST_DEVICE_UNKNOWN     # Desconhecido, offline
  - 1 - AST_DEVICE_NOT_INUSE   # Disponível
  - 2 - AST_DEVICE_INUSE       # Ocupado
  - 3 - AST_DEVICE_BUSY        # Ocupado
  - 4 - AST_DEVICE_INVALID     # Inválido
  - 5 - AST_DEVICE_UNAVAILABLE # Indisponível
  - 6 - AST_DEVICE_RINGING     # Tocando
  - 7 - AST_DEVICE_RINGINUSE   # Tocando
  - 8 - AST_DEVICE_ONHOLD      # Ocupado

- Pausa
  - 0 - Não está em pausa
  - 1 - Em pausa

* **RETORNO**

```json
{
  "id": 1,
  "nome": "Teste",
  "status": 1,
  "ramal": "SIP/1005",
  "fila": "Atendimento",
  "fila_padrao": "Atendimento",
  "pausa": 1,
  "pausa_motivo": "Intervalo",
  "max_chat_atende": 5,
  "status_date": 1619723823684,
  "ligacoes_efetuadas": 1,
  "ligacoes_recebidas": 1,
  "ligacoes_naoatendidas": 0,
  "ligacoes_naocompletadas": 0,
  "ligacoes_ativas": []
}
```

#### AgenteLigacao
----
Evento emitido quando o agente recebe, faz ou finaliza uma ligação.

- Sentido
  - 1 - Entrante
  - 2 - Sainte

- Sattus
  - 1 - Tocando
  - 2 - Ativa
  - 3 - Finalizada
  - 4 - Nao_Atendida
  - 5 - Nao_Completada

* **RETORNO**

```json
{
  "callid": "1619723759.173",
  "inicio": 1619723756751,
  "fila": "Atendimento",
  "agente_id": 1,
  "agente_nome": "Teste",
  "agente_ramal": "SIP/1005",
  "status": 3,
  "sentido": 2,
  "numero": "982332366",
  "atendida": 1619723770425,
  "fim": 1619723823684
}
```
## Metodos
Todos os métodos que podem ser chamados via Websocket são async com callback, então implemente os seus callback. Alguns métodos podem produzir eventos que serão enviados para todos que monitoram a fila em questão mas a maioria serão enviados apenas para quem chamou o médoto.

* Exemplo: Chamando método `login_agente` e implementando um callback chamado `resposta`

```js
const client = require("socket.io-client")("https://localhost:3000", {
    "rejectUnauthorized": false
});

client.emit('login_agente', { nome: 'agente1', senha: '*****' }, resposta => {
    console.log(`Login resposta: ${JSON.stringify(resposta)}`);
    if (resposta.error) {
        console.log(`Verifique os dados de acesso`);
    }
    else {
        // Login ok
        console.log('Dae sim...logado');
    }
});
```

#### AdicionarAgente
----
Loga um agente em uma determinada fila

* **PARAMETROS**
 * `ramal` (String): Ramal do agente, ex: SIP/1001
 * `fila` (String): Nome da fila
 * `pausado` (Integer): `0` Não pausado, `1` Pausado
 * `nome` (String) Nome do agente

* **RETORNO**

Em caso de sucesso.
```json
{
  "response": "Success",
  "actionid": "2",
  "message": "Added interface to queue",
  "events": [],
  "ramal": "SIP/1001",
  "fila": "Atendimento"
}
```

Em caso de Erro

```json
{
  "response": "Error",
  "actionid": "3",
  "message": "Unable to add interface: Already there",
  "events": [],
  "ramal": "SIP/1001",
  "fila": "Atendimento"
}
```

* Exemplo

```js
client.emit('AdicionarAgente', { ramal: 'SIP/1001', fila: 'Atendimento', pausado: 0, nome: 'Agente1' }, resposta => {
    console.log(`AdicionarAgente resposta: ${JSON.stringify(resposta, null, 2)}`);    
});
```

#### AdicionarFilaAgente
----
Adiciona uma nova fila ao cadastro de filas do agente.

* **PARAMETROS**
 * `fila` (String): Nome da fila
 * `id_agente` (Integer): Id do agente

* **RETORNO**

Em caso de sucesso.
```json
{
  "id_agente": 1,
  "fila": "Fila2",
  "id": 168
}
```

Em caso de Erro

```json
 {
  "error": true,
  "msg": "Falha ao adicinar a fila ao agente"
}
```

* Exemplo

```js
client.emit('AdicionarFilaAgente', { fila: 'Atendimento', id_agente: 1 }, resposta => {
    console.log(`Resposta: ${JSON.stringify(resposta, null, 2)}`);    
});
```

#### Conferencia
----
Adiciona um destino a uma sala de conferência. O sistema disca para o destino e o coloca na sala de conferência informada, se a sala não existir ela será criada.

* **PARAMETROS**
 * `destino` (String): número de destino
 * `rota` (String): Rota a ser usada
 * `sala` (String): Nome da sala

* **RETORNO**

Em caso de sucesso.
```json
{
  "response": "Success",
  "actionid": "4",
  "message": "Originate successfully queued",
  "events": [],
  "callid": 1620050709280
}
```

Em caso de Erro

```json
{
  "response": "Error",
  "actionid": "3",
  "message": "Unable to add interface: Already there",
}
```

* Exemplo

```js
client.emit('Conferencia', { destino: '123456', rota: 'Ramais', sala: 'Conf1' }, resposta => {
    console.log(`Resposta: ${JSON.stringify(resposta, null, 2)}`);    
});
```

#### Desligar
----
Desliga uma ligação por número ou canal. O número pode ser o número do ramal com (`SIP/Numero`) ou número de destino

* **PARAMETROS por número**
 * `destino` (String): número de destino

* **PARAMETROS por canal**
 * `canal` (String): Canal da ligação

* **RETORNO**

Em caso de sucesso.
```json
{
  "response": "Success",
  "actionid": "7",
  "eventlist": "start",
  "message": "Channels hung up will follow",
  "events": [
    {
      "lines": [
        "Event: ChannelsHungupListComplete",
        "ActionID: 7",
        "EventList: Complete",
        "ListItems: 0"
      ],
      "EOL": "\r\n",
      "variables": {},
      "event": "ChannelsHungupListComplete",
      "actionid": "7",
      "eventlist": "Complete",
      "listitems": "0"
    }
  ]
}
```

Em caso de Erro

```json
{
  "response": "Error",
  "actionid": "9",
  "message": "No such channel",
  "events": []
}
```

* Exemplo

```js
client.emit('Desligar', { destino: 'SIP/1001' }, resposta => {
    console.log(`Resposta: ${JSON.stringify(resposta, null, 2)}`);    
});
```

#### Discar
----
Disca para uma origem (`Ramal`) e destino, quando o ramal atender liga para o destino.

* **PARAMETROS**
 * `origem` (String): Ramal de origem
 * `destino` (String): número de destino
 * `rota` (String): Rota a ser usada

* **RETORNO**

Em caso de sucesso.
```json
{
  "response": "Success",
  "actionid": "11",
  "message": "Originate successfully queued",
  "events": [],
  "callid": 1620052250949
}
```

Em caso de Erro

```json
{
  "response": "Error",
  "actionid": "13",
  "message": "Invalid channel",
  "events": [],
  "callid": 1620052667081
}
```

* Exemplo

```js
client.emit('Discar', { origem: 'SIP/1005', destino: '123', rota: 'Ramais' }, resposta => {
    console.log(`Resposta: ${JSON.stringify(resposta, null, 2)}`);    
});
```

#### getFilas
----
Obtem uma lista das filas que o usuário tem acesso.

* **PARAMETROS**
 * `fila` (Opcional, String, Array): Filtra por uma ou mais filas

* **RETORNO**

Em caso de sucesso.
```json
[
    {
        "name": "Nome_da_fila",
        "id": 1,
        ...
    }
]
```

Em caso de Erro

```json
{
  "error": true,
  "errorDetails": "You do not have access to this queue."
}
```

* Exemplo

```js
client.emit('getFilas', null, resposta => {
    console.log(`Resposta: ${JSON.stringify(resposta, null, 2)}`);    
});
```

#### getAgentes
----
Obtem uma lista de agentes das filas que o usuário tem acesso.

* **PARAMETROS**
 * `fila` (Opcional, String, Array): Filtra por uma ou mais filas

* **RETORNO**

Em caso de sucesso.
```json
[
    {
        "id": 1,
        "nome": "Teste",
        "status": 1,
        "ramal": "SIP/1005",
        "fila": "Fila1",
        "fila_padrao": "Fila1",
        "pausa": 0,
        "pausa_motivo": "",
        "max_chat_atende": 5,
        "status_date": 1620053475574,
        "ligacoes_efetuadas": 0,
        "ligacoes_recebidas": 0,
        "ligacoes_naoatendidas": 0,
        "ligacoes_naocompletadas": 0,
        "filas": [
            "Fila1",
            "Fila2"
        ]
    }
]
```

Em caso de Erro

```json
{
  "error": true,
  "errorDetails": "You do not have access to this queue."
}
```

* Exemplo

```js
client.emit('getAgentes', null, resposta => {
    console.log(`Resposta: ${JSON.stringify(resposta, null, 2)}`);    
});
```

#### PausarAgente
----
Coloca um agente em pausa em todas as filas que ele estiver logado.

* **PARAMETROS**
 * `ramal` (String): Ramal do agente. Ex: SIP/1001
 * `motivo` (String): Motivo da pausa

* **RETORNO**

Em caso de sucesso.
```json
{
  "response": "Success",
  "actionid": "2",
  "message": "Interface paused successfully",
  "events": [],
  "ramal": "SIP/1005"
}
```

Em caso de Erro

```json
{
  "response": "Error",
  "actionid": "4",
  "message": "Interface not found",
  "events": [],
  "ramal": "SIP/1006"
}
```

* Exemplo

```js
client.emit('PausarAgente', { ramal: 'SIP/1001', motivo: 'Intervalo' }, resposta => {
    console.log(`Resposta: ${JSON.stringify(resposta, null, 2)}`);    
});
```

#### RemoverAgente
----
Desloga o agente da fila.

* **PARAMETROS**
 * `ramal` (String): Ramal do agente. Ex: SIP/1001
 * `fila` (String): Nome da fila.

* **RETORNO**

Em caso de sucesso.
```json
{
  "response": "Success",
  "actionid": "5",
  "message": "Removed interface from queue",
  "events": [],
  "ramal": "SIP/1005",
  "fila": "Whats01"
}
```

Em caso de Erro

```json
{
  "response": "Error",
  "actionid": "6",
  "message": "Unable to remove interface: Not there",
  "events": [],
  "ramal": "SIP/1005",
  "fila": "Whats01"
}
```

* Exemplo

```js
client.emit('RemoverAgente', { ramal: 'SIP/1001', motivo: 'Fila1' }, resposta => {
    console.log(`Resposta: ${JSON.stringify(resposta, null, 2)}`);    
});
```

#### RemoverFila
----
Remove uma fila. Remove a fila do banco de dados e notifica os agentes.

* **PARAMETROS**
 * `fila` (String): Nome da fila.

* **RETORNO**

Em caso de sucesso.
```json
{
  "name": "Nome da fila",
  "id": id_da_fila,
  ...
}
```

Em caso de Erro

```json
{
  "error": true,
  "errorDetails": "You do not have access to this queue."
}
```

* Exemplo

```js
client.emit('RemoverFila', { fila: 'Fila2' }, resposta => {
    console.log(`Resposta: ${JSON.stringify(resposta, null, 2)}`);    
});
```

#### RemoverFilaAgente
----
Remove uma fila do cadastro do agente e notifica ele.

* **PARAMETROS**
 * `fila` (String): Nome da fila.
 * `id_agente` (Integer): ID do agente.

* **RETORNO**

Em caso de sucesso.
```json
[
  {
    "id": 168,
    "id_agente": 1,
    "fila": "Fila2"
  }
]
```

Em caso de Erro

```json
{
  "error": true,
  "errorDetails": "You do not have access to this queue."
}
```

* Exemplo

```js
client.emit('RemoverFilaAgente', { fila: 'Fila2', id_agente: 1 }, resposta => {
    console.log(`Resposta: ${JSON.stringify(resposta, null, 2)}`);    
});
```

#### SyncFila
----
Recarrega as configurações de uma fila. Use depois de mudar algo no cadastro da fila.

* **PARAMETROS**
 * `fila` (String): Nome da fila.

* **RETORNO**

Em caso de sucesso.
```json
{
  "name": "Nome da fila",
  "id": id_da_fila,
  ...
}
```

Em caso de Erro

```json
{
  "error": true,
  "errorDetails": "You do not have access to this queue."
}
```

* Exemplo

```js
client.emit('SyncFila', { fila: 'Fila2', id_agente: 1 }, resposta => {
    console.log(`Resposta: ${JSON.stringify(resposta, null, 2)}`);    
});
```

#### TirarPausaAgente
----
Tira a pausa do agente.

* **PARAMETROS**
 * `ramal` (String): Ramal do agente

* **RETORNO**

Em caso de sucesso.
```json
{
  "response": "Success",
  "actionid": "9",
  "message": "Interface unpaused successfully",
  "events": [],
  "ramal": "SIP/1005"
}
```

Em caso de Erro

```json
{
  "error": true,
  "errorDetails": "You do not have access to this queue."
}
```

* Exemplo

```js
client.emit('TirarPausaAgente', { ramal: 'SIP/1005' }, resposta => {
    console.log(`Resposta: ${JSON.stringify(resposta, null, 2)}`);    
});
```

# online
Modulo responsável por receber eventos de outros serviços e publicá-los nos canais das filas.

## Login
Para publicar eventos via websocket os usuários precisam efetuar login no socket.

* `login` - Os Usuários devem chamar o método `login` informando o token de acesso conforme abaixo:

Exemplo:
```js
const client = require("socket.io-client")("https://localhost:3000/online", {
    "rejectUnauthorized": false
});

client.emit('login', { token: 'supertoken' }, resposta => {
    console.log(`Login resposta: ${JSON.stringify(resposta)}`);
    if (resposta.error) {
        console.log(`Verifique os dados de acesso`);
    }
    else {
        // Login ok
        console.log('Dae sim...logado');
    }
});
```

## Eventos
Para publicar um evento o único parametro obrigatório é a **fila**, exemplo:

Exemplo:
```js
client.emit('NovoAtendimento', { fila: 'Atendimento', de: '123', para: '321' }, resposta => {
    console.log(`NovoAtendimento resposta: ${JSON.stringify(resposta, null, 2)}`);
});
```

# Filas de chat e WhatsApp
As seguintes variáveis estão disponíveis em filas de chat e WhatsApp
* `MsgEnviadas` - Contador de mensagens enviadas
* `MsgRecebidas` - Contador de mensagens recebidas
* `TotalAtendimentosEntrantes` - Contador de atendimentos criados pelos clientes (Recebidos/Entrantes)
* `TotalAtendimentosSaintes` - Contador de atendimentos criados pelos agentes (Saintes)
* `TotalAtendimentosEncerrados` - Contador de atendimentos encerrados
* `AtendimentosAguardando` - Array com os atendimentos aguardando atendimento na fila.
O Array `AtendimentosAguardando` possui a seguinte estrutura.
```json
{
 "fila": "NOME_DA_FILA",
  "data": "TIMESTAMP",
  "atendimentoId": "ID_DO_ATENDIMENTO",
}
```
## Eventos de filas de chat e WhatsApp
Da mesma forma que os eventos de filas de voz os eventos de filas de chat e WhatsApp são publicados nas salas das filas e são enviados para os agentes e supervisores que tiverem acesso a fila depois de chamarem o método `monitorar_filas`.

* Como receber os eventos das filas: Chame o método `monitorar_filas`

```js
client.emit('monitorar_filas', {}, resposta => {
    console.log(`monitorar_filas resposta: ${JSON.stringify(resposta)}`);
});
```

#### FilaAtendimentoAguardandoAdicionado
----
Evento emitido quando um atendimento é adicionado no aguardando da fila.

* **RETORNO**

Dados do atendimento
```json
{
 "fila": "NOME_DA_FILA",
  "data": "TIMESTAMP",
  "atendimentoId": "ID_DO_ATENDIMENTO",
}
```

#### FilaAtendimentoAguardandoRemovido
----
Evento emitido quando um atendimento é removido do aguardando da fila.

* **RETORNO**

Dados do atendimento removido
```json
{
  "fila": "NOME_DA_FILA",
  "data": "TIMESTAMP",
  "atendimentoId": "ID_DO_ATENDIMENTO",
  "agente":"NOME_AGENTE",
  "tipo":"transferencia || encerramento || atendido"
}
```

#### FilaNovoAtendimento
----
Evento emitido quando um novo atendimento é criado.

* **RETORNO**

Dados do atendimento
```json
{
  "fila": "NOME_DA_FILA",
  "sentido": "Entrante || Sainte",
  "atendimentoId": "ID_DO_ATENDIMENTO"
}
```

#### FilaAtendimentoEncerrado
----
Evento emitido quando um atendimento é encerrado.

* **RETORNO**

Dados do atendimento
```json
{
  "fila": "NOME_DA_FILA",
  "data": "TIMESTAMP",
  "atendimentoId": "ID_DO_ATENDIMENTO",
  "agente":"NOME_AGENTE",
  "tipo":"encerramento"
}
```

#### FilaNovaMensagem
----
Evento emitido quando uma nova mensagem é enviada ou recebida.

* **RETORNO**

Dados do atendimento
```json
{
  "fila": "NOME_DA_FILA",
  "sentido": "entrante || sainte",
  "atendimentoId": "ID_DO_ATENDIMENTO",
  "agente":"NOME_AGENTE"
}
```