const log = require("node-live-log")("ami/Metodos/Agente");
const { ami } = require('../../../index')
const QueueEvents = require('xcontact-lib/models/QueueEvents');

const QueueAgenteLogin = require('../../../util/QueueAgenteLogin')
const QueueAgenteLogoff = require('../../../util/QueueAgenteLogoff')

exports.getAgentes = async (socket, request) => {
    let filas = ami.getFilas().filter(f => socket.filas.includes(f.nome));
    if (request && request.fila) {
        if (Array.isArray(request.fila)) {
            filas = filas.filter(f => request.fila.includes(f.nome));
        }
        else {
            filas = filas.filter(f => f.nome == request.fila);
        }
    }
    let agentes = [];
    for (let i = 0; i < filas.length; i++) {
        for (let j = 0; j < filas[i].agentes.length; j++) {
            let ag = JSON.parse(JSON.stringify(filas[i].agentes[j]));
            const found = agentes.find(a => a.id == ag.id);
            if (!found) {
                ag.filas = [ag.fila];
                agentes.push(ag);
            }
            else {
                // Nao repetir filas
                const repetida = found.filas.find(f => f.name == ag.fila);
                if (!repetida) {
                    found.filas.push(ag.fila);
                }
            }
        }
    }
    return agentes;
}

exports.AdicionarAgente = async (socket, request) => {
    log.debug('AdicionarAgente REQUEST: ', request)
    const ret = await ami.asterConn.AdicionarAgente(request)
    log.debug('AdicionarAgente RESPONSE: ', ret)
    if (ret.response === 'Success') {
        try {
            const loginData = {
                event_start: new Date(),
                agent: request.nome,
                queue: request.fila
            }
            QueueAgenteLogin.addJob(loginData)
        } catch (error) {
            log.error('AdicionarAgente - Erro ao adicionar job: ', error)
        }
    }
    return ret
}

exports.RemoverAgente = async (socket, request) => {
    log.debug('RemoverAgente REQUEST: ', request)
    const ret = await ami.asterConn.RemoverAgente(request)
    log.debug('RemoverAgente RESPONSE: ', ret)
    if (ret.response === 'Success') {
        try {
            const agente = ami.getAgenteByRamal(request.ramal)            
            if (!agente) {
                log.error('Remover - Agente não encontrado ao salvar no banco para o ramal: ', request.ramal)
            } else {
                const logoffData = {
                    event_end: new Date(),
                    agent: agente.nome,
                    queue: request.fila
                }
                QueueAgenteLogoff.addJob(logoffData)
            }
        } catch (error) {
            log.error('RemoverAgente - Erro ao adicionar job: ', error)
        }
    }
    return ret
}

exports.PausarAgente = async (socket, request) => {
    log.debug('PausarAgente REQUEST: ', request)
    const ret = await ami.asterConn.PausarAgente(request)
    log.debug('PausarAgente RESPONSE: ', ret)
    if (ret.response === 'Success') {
        try {
            const agente = ami.getAgenteByRamal(request.ramal)            
            if (!agente) {
                log.error('PausarAgente - Agente não encontrado ao salvar no banco para o ramal: ', request.ramal)
            } else {
                // log.debug('PausarAgente - Agente encontrado: ', agente)
                await new QueueEvents({
                    event: 'Pause',
                    event_start: new Date(),
                    envent_group: 1,
                    agent: agente.nome,
                    event_data1: request.motivo
                }).save()
            }
        } catch (error) {
            log.error('PausarAgente - Erro ao salvar no banco: ', error)
        }
    }
    return ret
}

exports.TirarPausaAgente = async (socket, request) => {
    log.debug('TirarPausaAgente REQUEST: ', request)
    const ret = await ami.asterConn.TirarPausaAgente(request)
    log.debug('TirarPausaAgente RESPONSE: ', ret)
    if (ret.response === 'Success') {
        try {
            const agente = ami.getAgenteByRamal(request.ramal)
            if (!agente) {
                log.error('TirarPausaAgente - Agente não encontrado ao salvar no banco para o ramal: ', request.ramal)
            } else {
                await QueueEvents.query.require(false)
                .where(QueueEvents.where.gte('event_start', new Date(new Date().setDate(new Date().getDate() - 5))))
                .where('event', 'Pause')
                .where('event_end', null)
                .where('agent', agente.nome)
                .update({ event_end: new Date() })
            }
        } catch (error) {
            log.error('TirarPausaAgente - Erro ao salvar no banco: ', error)
        }
    }
    return ret
}