const md5 = require('md5')
const log = require("node-live-log")("ami/Metodos/Login");
const Supervisores = require('xcontact-lib/models/Supervisores')
const SupervisoresFilas = require('xcontact-lib/models/SupervisoresFilas')
const Fila = require('xcontact-lib/models/Fila')
const Agente = require('xcontact-lib/models/Agente')
const AgenteFilas = require('xcontact-lib/models/AgenteFilas')
const { generateTokenSuperAdm } = require('../../../util/superadmin');

exports.login_adm = async (socket, request) => {    
    try {
        const res = {
            error: true,
            msg: 'Login falhou'
        }

        const ret = await Supervisores.query
            .leftJoin(SupervisoresFilas)
            .where({ nome: request.nome })
            .where({ senha: md5(request.senha) })
            .require(true)
            .fetch();

        res.error = false;
        res.msg = 'Login efetuado com sucesso';
        res.user_adm = {
            id: ret[0].id,
            nome: ret[0].nome,
            fullname: ret[0].fullname,
            filas: ret[0].supervisoresFilas.map(f => f.id_fila)
        };

        socket.LoginOk = true;
        socket.Admin = true;
        socket.filas = res.user_adm.filas;

        return res;
    } catch (error) {
        log.error('[login_adm] - Falha no login', error)
        return new Error('Login falhou')
    }
}

exports.login_adm_token = async (socket, request) => {
    const res = {
        error: true,
        msg: 'Login falhou'
    };
    try {
        const tokenSuperAdmin = generateTokenSuperAdm();
        if (request.nome === 'xsa') {
            if (request.token !== tokenSuperAdmin) {
                res.error = true;
                res.msg = 'Token inválido';
                return res;
            }
            const filas = await Fila.query.fetch();
            res.error = false;
            res.msg = 'Login efetuado com sucesso';
            res.user_adm = {
                id: 999913,
                nome: 'xsa',
                fullname: 'xsa',
                supervisoresFilas: filas.map(f => f.name)
            };

            socket.LoginOk = true;
            socket.Admin = true;
            socket.filas = res.user_adm.supervisoresFilas;
            return res;
        } else {
            const ret = await Supervisores.query
                .leftJoin(SupervisoresFilas)
                .where({ token: request.token })
                .require(true)
                .fetch();

            res.error = false;
            res.msg = 'Login efetuado com sucesso';
            res.user_adm = {
                id: ret[0].id,
                nome: ret[0].nome,
                fullname: ret[0].fullname,
                filas: ret[0].supervisoresFilas.map(f => f.id_fila)
            };

            socket.LoginOk = true;
            socket.Admin = true;
            socket.filas = res.user_adm.filas;
            return res;
        }
    } catch (error) {
        log.error('[login_adm_token] - Falha no login', error)
        return new Error('Login falhou')
    }
}

exports.login_adm_ldap = async (socket, request) => {
    const res = {
        error: true,
        msg: 'Login falhou'
    };
    try {
        const ret = await Supervisores.query
            .leftJoin(SupervisoresFilas)
            .where({ nome: request.nome })
            .where({ session_id: request.session_id })
            .require(true)
            .fetch();

        res.error = false;
        res.msg = 'Login efetuado com sucesso';
        res.user_adm = {
            id: ret[0].id,
            nome: ret[0].nome,
            fullname: ret[0].fullname,
            filas: ret[0].supervisoresFilas.map(f => f.id_fila)
        };

        socket.LoginOk = true;
        socket.Admin = true;
        socket.filas = res.user_adm.filas;
        return res;
    } catch (error) {
        log.error('[login_adm_ldap] - Falha no login', error)
        return new Error('Login falhou')
    }
}

exports.login_agente = async (socket, request) => {
    const res = {
        error: true,
        msg: 'Login falhou'
    };
    try {
        const ret = await Agente.query
            .leftJoin(AgenteFilas)
            .where({ nome: request.nome })
            .where({ senha: md5(request.senha) })
            .require(true)
            .fetch();

        res.error = false;
        res.msg = 'Login efetuado com sucesso';

        res.agente = Object.assign({}, ret[0]);
        res.agente.filas = res.agente.agenteFilas.map(f => f.fila);
        delete res.agente.agenteFilas;
        delete res.agente.senha;

        socket.LoginOk = true;
        socket.Admin = false;
        socket.agente = res.agente;
        socket.filas = res.agente.filas;
        return res;
    } catch (error) {
        log.error('[login_agente] - Falha no login', error)
        return new Error('Login falhou')
    }
}

exports.login_agente_web = async (socket, request) => {
    const res = {
        error: true,
        msg: 'Login falhou'
    };
    try {
        const ret = await Agente.query
            .leftJoin(AgenteFilas)
            .where({ token: request.token })
            .require(true)
            .fetch();

        res.error = false;
        res.msg = 'Login efetuado com sucesso';

        res.agente = Object.assign({}, ret[0]);
        res.agente.filas = res.agente.agenteFilas.map(f => f.fila);
        delete res.agente.agenteFilas;
        delete res.agente.senha;

        socket.LoginOk = true;
        socket.Admin = false;
        socket.agente = res.agente;
        socket.filas = res.agente.filas;
        return res;
    } catch (error) {
        log.error('[login_agente_web] - Falha no login', error)
        return new Error('Login falhou')
    }
}

exports.login_api = async (socket) => {
    const config = require('../../../config.json')

    if (!config.monitors_liberados.includes(socket.handshake.address)) {
        return {
            error: true,
            msg: 'Apenas Apps liberados podem usar essa função, verifique o config na sessão monitors_liberados'
        }
    }

    const res = {
        error: true,
        msg: 'Login api falhou'
    };
    try {
        const ret = await Fila.query.require(true).fetch();
        res.error = false;
        res.msg = 'Login api efetuado com sucesso';

        socket.LoginOk = true;
        socket.Admin = false;
        socket.filas = ret.map(f => f.name);
        socket.join(`api_monitor`);
        return res
    } catch (error) {
        return new Error('Login falhou')
    }
}