const readline = require('readline');
const xserver = process.argv[2] ? process.argv[2] : "http://localhost:3000";

// Apenas para fins de teste
const rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout,
    prompt: 'AMI CLI> '
});


const client = require("socket.io-client")(xserver, {
    "rejectUnauthorized": false
});

client.on('connect', () => {
    console.log('Parece que conectou');


    client.emit('login_api', null, resposta => {
        console.log(`Login resposta: ${JSON.stringify(resposta)}`);
        if (resposta.error) {
            console.log(`Verifique os dados de acesso`);
        }
        else {
            // Login ok
            console.log('Dae sim...logado');

            // client.emit('monitorar_filas', {}, resposta => {
            //     console.log(`monitorar_filas resposta: ${JSON.stringify(resposta)}`);
            // });
        }
    });

});

client.on('disconnect', (e) => {
    console.log(e);
    console.log('Desconectou do servidor');
});

client.on('connect_timeout', (timeout) => {
    console.log('Desconectado, connection timeout.');
});
client.on('error', (error) => {
    console.log('Desculpe, algo deu errado.');
    console.log(error);
});
client.on('connect_error', (error) => {
    console.log('Erro de conexão. Tem certeza que o serviço do Xcontact está ativo?');
    console.log(error);
});
client.on('reconnecting', (attemptNumber) => {
    console.log(`Estou tentando reconectar. Tentativa: ${attemptNumber}`);
});
client.on('reconnect_failed', () => {
    console.log('Não estou conseguindo reconectar. Tem certeza que o serviço do Xcontact está ativo?');
});
client.on('unauthorized', (err) => {
    console.log("Falha na autenticação:", err.message);
});

const Eventos = [
    'FilaAtualizada',
    'FilaRemovida',
    'AgenteAdicionado',
    'AgenteRemovido',
    'AgenteStatus',
    'AgenteLigacao',
    'FilaLigacaoAdicionada',
    'FilaLigacaoAtendida',
    'FilaLigacaoAbandonou',
    'FilaLigacaoTransbordou',
    'FilaLigacaoTerminou',
    'FilaAdicionadaAgente',
    'FilaRemovidaAgente',

    'FilaAtendimentoAguardandoAdicionado',
    'FilaAtendimentoAguardandoRemovido',    
    'FilaNovoAtendimento',
    'FilaAtendimentoEncerrado',
    'FilaNovaMensagem',
];

Eventos.forEach(event => {
    console.log(`Iniciando monitor para evento ${event}`);
    client.on(event, dados => {
        console.log(event);
        console.log(JSON.stringify(dados, null, 2))
    })
});

rl.on('line', (line) => {
    let pars = line.trim().split(' ');

    switch (pars[0]) {
        case 'sair':
        case 'quit':
        case 'q':
            rl.close();
            break;
        case '':
            break;
        case 'debug':
            debugProcess();
            break;
        case 'AdicionarAgente':
            client.emit('AdicionarAgente', { ramal: pars[1], fila: pars[2], pausado: pars[3], nome: pars[4] }, resposta => {
                console.log(`AdicionarAgente resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        case 'RemoverAgente':
            client.emit('RemoverAgente', { ramal: pars[1], fila: pars[2] }, resposta => {
                console.log(`RemoverAgente resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        case 'PausarAgente':
            client.emit('PausarAgente', { ramal: pars[1], motivo: pars[2] }, resposta => {
                console.log(`PausarAgente resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        case 'TirarPausaAgente':
            client.emit('TirarPausaAgente', { ramal: pars[1] }, resposta => {
                console.log(`TirarPausaAgente resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        case 'Discar':
            client.emit('Discar', { origem: pars[1], destino: pars[2], rota: pars[3] }, resposta => {
                console.log(`Discar resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        case 'Conferencia':
            client.emit('Conferencia', { destino: pars[1], rota: pars[2], sala: pars[3] }, resposta => {
                console.log(`Conferencia resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        case 'DesligarNumero':
            client.emit('Desligar', { destino: pars[1] }, resposta => {
                console.log(`Desligar resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        case 'DesligarCanal':
            client.emit('Desligar', { canal: pars[1] }, resposta => {
                console.log(`Desligar resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        case 'SyncFila':
            client.emit('SyncFila', { fila: pars[1] }, resposta => {
                console.log(`SyncFila resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        case 'getFilas':
            client.emit('getFilas', { fila: pars[1] }, resposta => {
                console.log(`getFilas resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        case 'getAgentes':
            client.emit('getAgentes', { fila: pars[1] }, resposta => {
                console.log(`getAgentes resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        case 'RemoverFila':
            client.emit('RemoverFila', { fila: pars[1] }, resposta => {
                console.log(`Remover resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        case 'AdicionarFilaAgente':
            client.emit('AdicionarFilaAgente', { fila: pars[1], id_agente: pars[2] }, resposta => {
                console.log(`AdicionarFilaAgente resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        case 'RemoverFilaAgente':
            client.emit('RemoverFilaAgente', { fila: pars[1], id_agente: pars[2] }, resposta => {
                console.log(`RemoverFilaAgente resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        case 'AddLigacaoFila':
            client.emit('adicionarLigacaoNaFila', { 
                queue: pars[1], 
                linkedid: pars[2], 
                calleridnum: '123', 
                calleridname: 'Cliente 1', 
                position: 1, 
                count: 1 
            }, resposta => {
                console.log(`adicionarLigacaoNaFila resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        case 'abandonarLigacaoFila':
            client.emit('abandonarLigacaoNaFila', { 
                queue: pars[1], 
                linkedid: pars[2], 
                position: 1,
            }, resposta => {
                console.log(`abandonarLigacaoNaFila resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        case 'atenderLigacaoFila':
            client.emit('atenderLigacaoNaFila', { 
                queue: pars[1], 
                linkedid: pars[2], 
                membername: 'Bala', 
                interface: 'SIP/1000', 
                ringtime: 10
            }, resposta => {
                console.log(`atenderLigacaoNaFila resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        case 'desligarLigacaoFila':
            client.emit('desligarLigacaoNaFila', { 
                queue: pars[1], 
                linkedid: pars[2], 
                membername: 'Bala', 
                interface: 'SIP/1000', 
                reason: 'agent', 
                talktime: 30
            }, resposta => {
                console.log(`desligarLigacaoNaFila resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        default:
            break;
    }
    rl.prompt();
}).on('close', () => {
    console.log('Até logo!');
    process.exit(0);
});