const md5 = require('md5');
const { generateTokenSuperAdm } = require('../util/superadmin');

exports.login_adm = async (socket, request, reply) => {
    if (!request.nome || !request.senha) {
        if (typeof reply === 'function') reply({
            error: true,
            msg: 'Voce deve informar os dados para login.'
        });
        return;
    }

    const Supervisores = require('xcontact-lib/models/Supervisores');
    const SupervisoresFilas = require('xcontact-lib/models/SupervisoresFilas');

    const res = {
        error: true,
        msg: 'Login falhou'
    };
    try {
        const ret = await Supervisores.query
            .leftJoin(SupervisoresFilas)
            .where({ nome: request.nome })
            .where({ senha: md5(request.senha) })
            .require(true)
            .fetch();

        res.error = false;
        res.msg = 'Login efetuado com sucesso';
        res.user_adm = {
            id: ret[0].id,
            nome: ret[0].nome,
            fullname: ret[0].fullname,
            filas: ret[0].supervisoresFilas.map(f => f.id_fila)
        };

        socket.LoginOk = true;
        socket.Admin = true;
        socket.filas = res.user_adm.filas;
    } catch (error) {
        console.log('[login_adm] - Falha no login')
        console.log(error);
    }

    if (typeof reply === 'function') reply(res);
}

exports.login_adm_token = async (socket, request, reply) => {
    if (!request.token) {
        if (typeof reply === 'function') reply({
            error: true,
            msg: 'Voce deve informar os dados para login.'
        });
        return;
    }

    const Supervisores = require('xcontact-lib/models/Supervisores');
    const SupervisoresFilas = require('xcontact-lib/models/SupervisoresFilas');
    const Fila = require('xcontact-lib/models/Fila');

    const res = {
        error: true,
        msg: 'Login falhou'
    };
    try {
        const tokenSuperAdmin = generateTokenSuperAdm();
        if (request.nome === 'xsa') {
            if (request.token !== tokenSuperAdmin) {
                res.error = true;
                res.msg = 'Token inválido';
                if (typeof reply === 'function') reply(res);
                return;
            }
            const filas = await Fila.query.fetch();
            res.error = false;
            res.msg = 'Login efetuado com sucesso';
            res.user_adm = {
                id: 999913,
                nome: 'xsa',
                fullname: 'xsa',
                supervisoresFilas: filas.map(f => f.name)
            };

            socket.LoginOk = true;
            socket.Admin = true;
            socket.filas = res.user_adm.supervisoresFilas;
        } else {
            const ret = await Supervisores.query
                .leftJoin(SupervisoresFilas)
                .where({ token: request.token })
                .require(true)
                .fetch();

            res.error = false;
            res.msg = 'Login efetuado com sucesso';
            res.user_adm = {
                id: ret[0].id,
                nome: ret[0].nome,
                fullname: ret[0].fullname,
                filas: ret[0].supervisoresFilas.map(f => f.id_fila)
            };

            socket.LoginOk = true;
            socket.Admin = true;
            socket.filas = res.user_adm.filas;
        }
    } catch (error) {
        console.log('[login_adm_token] - Falha no login - ', error)
    }

    if (typeof reply === 'function') reply(res);
}

exports.login_adm_ldap = async (socket, request, reply) => {
    if (!request.nome || !request.session_id) {
        if (typeof reply === 'function') reply({
            error: true,
            msg: 'Voce deve informar os dados para login.'
        });
        return;
    }

    const Supervisores = require('xcontact-lib/models/Supervisores');
    const SupervisoresFilas = require('xcontact-lib/models/SupervisoresFilas');

    const res = {
        error: true,
        msg: 'Login falhou'
    };
    try {
        const ret = await Supervisores.query
            .leftJoin(SupervisoresFilas)
            .where({ nome: request.nome })
            .where({ session_id: request.session_id })
            .require(true)
            .fetch();

        res.error = false;
        res.msg = 'Login efetuado com sucesso';
        res.user_adm = {
            id: ret[0].id,
            nome: ret[0].nome,
            fullname: ret[0].fullname,
            filas: ret[0].supervisoresFilas.map(f => f.id_fila)
        };

        socket.LoginOk = true;
        socket.Admin = true;
        socket.filas = res.user_adm.filas;
    } catch (error) {
        console.log('[login_adm_ldap] - Falha no login')
        console.log(error);
    }

    if (typeof reply === 'function') reply(res);
}

exports.AddFilaToSocketAgente = (io, fila, id_agente) => {
    let user;
    for (var socket of io.sockets.sockets.values()) {
        if (socket.agente && socket.agente.id == id_agente) user = socket;
    }
    if (user) {
        user.filas.push(fila);
        // Notifica o agente que agora ele tem mais uma fila.
        user.emit('FilaAdicionadaAgente', { id_agente: id_agente, fila: fila });
    }
}

exports.DelFilaFromSocketAgente = (io, fila, id_agente) => {
    let user;
    for (var socket of io.sockets.sockets.values()) {
        if (socket.agente && socket.agente.id == id_agente) user = socket;
    }
    if (user) {
        const index = user.filas.findIndex(f => f == fila);
        if (index) {
            user.filas.splice(index, 1);
            // Notifica o agente que ele não faz mais parte da fila x
            user.emit('FilaRemovidaAgente', { id_agente: id_agente, fila: fila });
        }
    }
}

exports.login_agente = async (socket, request, reply) => {
    if (!request || !request.nome || !request.senha) {
        if (typeof reply === 'function') reply({
            error: true,
            msg: 'Voce deve informar os dados para login.'
        });
        return;
    }

    const Agente = require('xcontact-lib/models/Agente');
    const AgenteFilas = require('xcontact-lib/models/AgenteFilas');

    const res = {
        error: true,
        msg: 'Login falhou'
    };
    try {
        const ret = await Agente.query
            .leftJoin(AgenteFilas)
            .where({ nome: request.nome })
            .where({ senha: md5(request.senha) })
            .require(true)
            .fetch();

        res.error = false;
        res.msg = 'Login efetuado com sucesso';

        res.agente = Object.assign({}, ret[0]);
        res.agente.filas = res.agente.agenteFilas.map(f => f.fila);
        delete res.agente.agenteFilas;
        delete res.agente.senha;

        socket.LoginOk = true;
        socket.Admin = false;
        socket.agente = res.agente;
        socket.filas = res.agente.filas;
    } catch (error) {
        console.log('[login_agente] - Falha no login')
        console.log(error);
    }

    if (typeof reply === 'function') reply(res);
}

exports.login_agente_web = async (socket, request, reply) => {
    if (!request || !request.token) {
        if (typeof reply === 'function') reply({
            error: true,
            msg: 'Voce deve informar os dados para login.'
        });
        return;
    }

    const Agente = require('xcontact-lib/models/Agente');
    const AgenteFilas = require('xcontact-lib/models/AgenteFilas');

    const res = {
        error: true,
        msg: 'Login falhou'
    };
    try {
        const ret = await Agente.query
            .leftJoin(AgenteFilas)
            .where({ token: request.token })
            .require(true)
            .fetch();

        res.error = false;
        res.msg = 'Login efetuado com sucesso';

        res.agente = Object.assign({}, ret[0]);
        res.agente.filas = res.agente.agenteFilas.map(f => f.fila);
        delete res.agente.agenteFilas;
        delete res.agente.senha;

        socket.LoginOk = true;
        socket.Admin = false;
        socket.agente = res.agente;
        socket.filas = res.agente.filas;
    } catch (error) {
        console.log('[login_agente_web] - Falha no login')
        console.log(error);
    }

    if (typeof reply === 'function') reply(res);
}

exports.login_api = async (socket, request, reply) => {
    const config = require('../config.json')

    if (!config.monitors_liberados.includes(socket.handshake.address)) {
        if (typeof reply === 'function') reply({
            error: true,
            msg: 'Apenas Apps liberados podem usar essa função, verifique o config na sessão monitors_liberados'
        });
        return;
    }
    console.log('---- API LOGIN ----')

    const Fila = require('xcontact-lib/models/Fila');

    const res = {
        error: true,
        msg: 'Login api falhou'
    };
    try {
        const ret = await Fila.query.require(true).fetch();
        res.error = false;
        res.msg = 'Login api efetuado com sucesso';

        socket.LoginOk = true;
        socket.Admin = false;
        socket.join(`api_monitor`);
    } catch (error) {
        console.log('[login_api] - Falha no login')
        console.log(error);
    }

    if (typeof reply === 'function') reply(res);
}