const dotenv = require('dotenv');
const config = require('./config.json');
dotenv.config();

if (!process.env.PGHOST) process.env.PGHOST = config.database.host;
if (!process.env.PGPASS) process.env.PGPASS = config.database.pass;
if (!process.env.AMI_HOST) process.env.AMI_HOST = config.asterisk.host;
if (!process.env.XCLOG_LEVEL) process.env.XCLOG_LEVEL = config.log.level;
if (!process.env.XCLOG_FILE) process.env.XCLOG_FILE = config.log.file;
if (!process.env.LOG_PORT) process.env.LOG_PORT = 6666;
if (!process.env.LOG_DATE_FORMAT) process.env.LOG_DATE_FORMAT = 'DD-MM-YYYY HH:mm:ss.SSS';


const wsport = process.env.PORT ? process.env.PORT : config.server.port;
const cron = require('node-cron')

const log = require("node-live-log")("index.js");
const Ami = require('./Ami');
const Online = require('./online')

const httpServer = require("http").createServer();
const io = require("socket.io")(httpServer);

// Ami start
const ami = new Ami();
ami.start();
exports.ami = ami

const { MethodRoute } = require('ioroute')
const methodRoute = new MethodRoute()
methodRoute.register('ami/Metodos')

io.on('connection', (socket) => {
    methodRoute.start(socket)
});

// const io = require('socket.io')(wsport);
httpServer.listen(wsport, '0.0.0.0');
log.info('Server listening at port ', wsport);

// Modulo online
const online = new Online(io)
online.start()

// Handlers
// const Login = require('./handlers/Login');

// const ioRoute = require('./util/ioRoute');

// Metodos midrados para o ioroute no diretorio ami/Metodos
// io.on('connection', (socket) => {
//     log.info(`Nova conexao, IP: ${socket.handshake.address}, SocketID: ${socket.id}`);

//     /**
//      * Resposta de ping
//      */
//     socket.on('ping', (request, reply) => {
//         log.info(`[ping] - Recebi ${JSON.stringify(request)}`);
//         if (typeof reply === 'function') reply('Pong');
//     });

//     /**
//      * Efetua login de usuários adm
//      */
//     socket.on('login_adm', (request, reply) => {
//         log.info(`[login_adm] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou login_adm`);

//         Login.login_adm(socket, request, reply);
//     });

//     // Login de supervisores via token
//     socket.on('login_adm_token', (request, reply) => {
//         log.info(`[login_adm_token] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou login_adm_token`);

//         Login.login_adm_token(socket, request, reply);
//     });

//     /**
//     * Efetua login de usuários adm via LDAP
//     */
//     socket.on('login_adm_ldap', (request, reply) => {
//         log.info(`[login_adm] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou login_adm_ldap`);

//         Login.login_adm_ldap(socket, request, reply);
//     });

//     /**
//      * Efetua login de agentes
//      */
//     socket.on('login_agente', (request, reply) => {
//         log.info(`[login_agente] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou login_agente`);

//         Login.login_agente(socket, request, reply);
//     });

//     /**
//     * Efetua login de agentes via token
//     */
//     socket.on('login_agente_web', (request, reply) => {
//         log.info(`[login_agente_web] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou login_agente`);

//         Login.login_agente_web(socket, request, reply);
//     });

//     /**
//      * Efetua login de apis locais
//      */
//     socket.on('login_api', (request, reply) => {
//         log.info(`[login_api] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou login_api`);

//         Login.login_api(socket, request, reply);
//     });

//     /**
//      * Monitora as filas que o usuário tem acesso
//      */
//     socket.on('monitorar_filas', (request, reply) => {
//         log.info(`[monitorar_filas] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou monitorar_filas`);

//         const route = new ioRoute(socket, request, reply);
//         route.needQueue = true;
//         route.handler(() => {
//             socket.filas.forEach(fila => {
//                 socket.join(`fila_${fila}`);
//             });
//             return { error: false, msg: 'Monitor iniciando', filas: socket.filas }
//         });
//     });


//     /**
//      * Obtem os dados das filas que o usuário tem acesso.
//      */
//     socket.on('getFilas', (request, reply) => {
//         log.info(`[getFilas] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou getFilas`);

//         const route = new ioRoute(socket, request, reply);
//         route.needQueue = true;
//         route.handler(() => {
//             let filas = ami.getFilas().filter(f => socket.filas.includes(f.nome));
//             if (request && request.fila) {
//                 if (Array.isArray(request.fila)) {
//                     filas = filas.filter(f => request.fila.includes(f.nome));
//                 }
//                 else {
//                     filas = filas.filter(f => f.nome == request.fila);
//                 }
//             }
//             return filas;
//         });
//     });

//     socket.on('getAgentes', (request, reply) => {
//         log.info(`[getAgentes] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou getAgentes`);

//         const route = new ioRoute(socket, request, reply);
//         route.needQueue = true;
//         route.handler(() => {
//             let filas = ami.getFilas().filter(f => socket.filas.includes(f.nome));
//             if (request && request.fila) {
//                 if (Array.isArray(request.fila)) {
//                     filas = filas.filter(f => request.fila.includes(f.nome));
//                 }
//                 else {
//                     filas = filas.filter(f => f.nome == request.fila);
//                 }
//             }
//             let agentes = [];
//             for (let i = 0; i < filas.length; i++) {
//                 for (let j = 0; j < filas[i].agentes.length; j++) {
//                     let ag = JSON.parse(JSON.stringify(filas[i].agentes[j]));
//                     const found = agentes.find(a => a.id == ag.id);
//                     if (!found) {
//                         ag.filas = [ag.fila];
//                         agentes.push(ag);
//                     }
//                     else {
//                         // Nao repetir filas
//                         const repetida = found.filas.find(f => f.name == ag.fila);
//                         if (!repetida) {
//                             found.filas.push(ag.fila);
//                         }
//                     }
//                 }
//             }
//             return agentes;
//         });
//     });

//     socket.on('AdicionarAgente', (request, reply) => {
//         log.info(`[AdicionarAgente] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou AdicionarAgente`);

//         const route = new ioRoute(socket, request, reply);
//         route.addValidate('ramal');
//         route.addValidate('fila');
//         route.addValidate('nome');
//         route.handler(async () => await ami.asterConn.AdicionarAgente(request));
//     });

//     socket.on('RemoverAgente', (request, reply) => {
//         log.info(`[RemoverAgente] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou RemoverAgente`);
//         const route = new ioRoute(socket, request, reply);
//         route.addValidate('ramal');
//         route.addValidate('fila');
//         route.handler(async () => {
//             resp = await ami.asterConn.RemoverAgente(request);
//             log.info('RemoverAgente retornou...');
//             log.info(resp);
//             return resp;
//         });
//     });

//     socket.on('PausarAgente', (request, reply) => {
//         log.info(`[PausarAgente] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou PausarAgente`);

//         const route = new ioRoute(socket, request, reply);
//         route.addValidate('ramal');
//         route.handler(async () => await ami.asterConn.PausarAgente(request));
//     });

//     socket.on('TirarPausaAgente', (request, reply) => {
//         log.info(`[TirarPausaAgente] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou TirarPausaAgente`);

//         const route = new ioRoute(socket, request, reply);
//         route.addValidate('ramal');
//         route.handler(async () => await ami.asterConn.TirarPausaAgente(request));
//     });

//     socket.on('Discar', (request, reply) => {
//         log.info(`[Discar] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou Discar`);

//         const route = new ioRoute(socket, request, reply);
//         route.addValidate('origem');
//         route.addValidate('destino');
//         route.addValidate('rota');
//         route.needQueue = false;
//         route.handler(async () => await ami.asterConn.Discar(request));
//     });

//     socket.on('Conferencia', (request, reply) => {
//         log.info(`[Conferencia] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou Conferencia`);

//         const route = new ioRoute(socket, request, reply);
//         route.addValidate('sala');
//         route.addValidate('destino');
//         route.addValidate('rota');
//         route.needQueue = false;
//         route.handler(async () => await ami.asterConn.Conferencia(request));
//     });

//     socket.on('Desligar', (request, reply) => {
//         log.info(`[Desligar] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou Desligar`);

//         const route = new ioRoute(socket, request, reply);
//         route.needQueue = false;
//         route.handler(async () => await ami.asterConn.Desligar(request));
//     });

//     socket.on('SyncFila', (request, reply) => {
//         log.info(`[SyncFila] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou SyncFila`);

//         const route = new ioRoute(socket, request, reply);
//         route.addValidate('fila');
//         route.admin = true;
//         route.handler(async () => await ami.syncFila(request));
//     });

//     socket.on('RemoverFila', (request, reply) => {
//         log.info(`[RemoverFila] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou RemoverFila`);

//         const route = new ioRoute(socket, request, reply);
//         route.addValidate('fila');
//         route.admin = true;
//         route.handler(async () => await ami.RemoverFila(request));
//     });

//     socket.on('AdicionarFilaAgente', (request, reply) => {
//         log.info(`[AdicionarFilaAgente] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou AdicionarFilaAgente`);

//         const route = new ioRoute(socket, request, reply);
//         route.addValidate('fila');
//         route.addValidate('id_agente');
//         route.admin = true;
//         route.handler(async () => {
//             let resp = await ami.AdicionarFilaAgente(request);
//             if (resp) {
//                 Login.AddFilaToSocketAgente(io, request.fila, request.id_agente);
//                 return resp;
//             }
//             else {
//                 return { error: true, msg: "Falha ao adicinar a fila ao agente" };
//             }
//         });
//     });

//     socket.on('RemoverFilaAgente', (request, reply) => {
//         log.info(`[RemoverFilaAgente] - Cliente no IP ${socket.handshake.address}, SocketID: ${socket.id} solicitou RemoverFilaAgente`);

//         const route = new ioRoute(socket, request, reply);
//         route.addValidate('fila');
//         route.addValidate('id_agente');
//         route.admin = true;
//         route.handler(async () => {
//             let resp = await ami.RemoverFilaAgente(request);
//             if (resp) {
//                 Login.DelFilaFromSocketAgente(io, request.fila, request.id_agente);
//                 return resp;
//             }
//             else {
//                 return { error: true, msg: "Falha ao remover a fila ao agente" };
//             }
//         });
//     });

//     socket.on('disconnect', (reason) => {
//         log.info(`Desconectou, IP: ${socket.handshake.address}, SocketID: ${socket.id}, Motivo: ${reason}`);
//     });
// });

// Evento inicial
ami.on('Iniciado', ini => {
    log.info(`Conectado ao servidor ${process.env.AMI_HOST}, WS rodando na porta ${wsport}`);
    //log.info(JSON.stringify(ini, null, 2));
})



// Eventos publicados nas salas das filas
const EventosBroadCast = [
    'FilaAtualizada',
    'FilaRemovida',
    'FilaLigacaoAdicionada',
    'FilaLigacaoAtendida',
    'FilaLigacaoAbandonou',
    'FilaLigacaoTransbordou',
    'FilaLigacaoTerminou',
    'AgenteAdicionado',
    'AgenteRemovido',
    'AgenteStatus',
    'AgenteLigacao',
    'FilaAdicionadaAgente',
    'FilaRemovidaAgente',
    'TransferenciaDireta',
    'TransferenciaConsulta'
];

/**
 * Enviando os eventos para devidas filas
 */
EventosBroadCast.forEach(event => {
    log.info(`Iniciando Online broadcast para evento ${event}`);
    ami.on(event, retorno => {
        // Publica todos os eventos na sala de monitoramento
        io.to('api_monitor').emit(event, retorno);

        let fila = retorno.fila;
        if (fila) {
            log.info(`Publicando evento ${event} na sala fila_${fila}`);
            io.to(`fila_${fila}`).emit(event, retorno);
        }
        else {
            log.info(`Olha tche, não tem fila no evento de fila, isso é interessante.`);
        }
    })
})

let reset_cron = "0 0 * * *"
if (config.reset_cron) {
    reset_cron = config.reset_cron
}
log.info(`Reset de statisticas agendado para ${reset_cron}`)
cron.schedule(reset_cron, () => {
    log.info('Rodando Reset de filas agendado')
    ami.reset()
})