const io = require('socket.io-client')

const client = io('ws://localhost:6666')

const readline = require('readline')
const fs = require('fs')

let FileLog = false
// const FileLog = 'my_log.txt'

function completer(line) {
    const completions = `help
q
quit
sair
enable file
enable error
enable info
enable warn
enable debug
disable file
disable error
disable info
disable warn
disable debug`
	.split(/\n/g)
    const hits = completions.filter((c) => c.startsWith(line));
    return [hits.length ? hits : completions, line];
}

const rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout,
    completer: completer,
    prompt: 'LOG CLI> '
})

client.on('connect', () => {
    console.log('Connected')
    rl.prompt(true);
})

// Listen a event log

const logInfo = (msg) => {
    console.info(msg)
    if(FileLog) {
        fs.appendFile(FileLog, msg);
    }
    rl.prompt(true)
}

client.on('log-info', logInfo)

const logWarn = (msg) => {
    console.warn(msg)
    if(FileLog) {
        fs.appendFile(FileLog, msg);
    }
    rl.prompt(true)
}

client.on('log-warn', logWarn)

const logError = (msg) => {
    console.error(msg)
    if(FileLog) {
        fs.appendFile(FileLog, msg);
    }
    rl.prompt(true)
}

client.on('log-error', logError)

const logDebug = (msg) => {
    console.debug(msg)
    if(FileLog) {
        fs.appendFile(FileLog, msg);
    }
    rl.prompt(true)
}

client.on('log-debug', logDebug)

rl.on('line', (line) => {
    let pars = line.trim().split(' ');

    switch (pars[0]) {
        case 'sair':
        case 'quit':
        case 'q':
            rl.close()
            break
        case '':
            break
        case 'help':
            console.log(`
help            --- Exibe essa lista
q               --- Sai deste cliente
quit            --- Alias para q
sair            --- Alias para q
enable file     --- Ativa gravação dos logs em arquivo
enable error    --- Ativa logs de Erro
enable info     --- Ativa logs de info
enable warn     --- Ativa logs de wanr
enable debug    --- Ativa logs de debug
disable file    --- Desativa gravação dos logs em arquivo
disable error   --- Desativa logs de erro
disable info    --- Desativa logs de info
disable warn    --- Desativa logs de warn
disable debug   --- Desativa logs de debug
`)
            break
        case 'disable':
            switch (pars[1]) {
                case 'file':
                    FileLog = false
                    console.log('Registro de log em arquivo DESATIVADO')
                    rl.prompt()
                    break
                case 'error':
                    client.removeListener('log-error', logError)
                    console.log('log-error disabled')
                    rl.prompt();
                    break
                case 'debug':
                    client.removeListener('log-debug', logDebug)
                    console.log('log-debug disabled')
                    rl.prompt();
                    break
                case 'info':
                    client.removeListener('log-info', logInfo)
                    console.log('log-info disabled')
                    rl.prompt();
                    break
                case 'warn':
                    client.removeListener('log-warn', logWarn)
                    console.log('log-warn disabled')
                    rl.prompt();
                    break
                default:
                    break
            }
        break
        case 'enable':
            switch (pars[1]) {
                case 'file':
                    FileLog = 'live-log.log'
                    console.log('Log sendo gravado em live-log.log')
                    rl.prompt()
                    break
                case 'error':
                    client.on('log-error', logError)
                    console.log('log-error enable')
                    rl.prompt();
                    break
                case 'debug':
                    client.on('log-debug', logDebug)
                    console.log('log-debug enable')
                    rl.prompt();
                    break
                case 'info':
                    client.on('log-info', logInfo)
                    console.log('log-info enable')
                    rl.prompt();
                    break
                case 'warn':
                    client.on('log-warn', logWarn)
                    console.log('log-warn enable')
                    rl.prompt();
                    break
                default:
                    break
            }
        break
        default:
            break
    }
    rl.prompt();
}).on('close', () => {
    console.log('Bye!');
    process.exit(0);
});