module.exports = class AgenteAmi {
    /**
    * @typedef agente
    * @type {object}
    * @property {number} id - Identificador do Agente
    * @property {string} nome - Nome do agente
    * @property {string} ramal - Ramal do agente
    * @property {string} fila - Fila do agente
    * @property {string} fila_padrao - Fila utilizada para ligações saintes.
    * @property {number} pausa - Indica se o agente está em pausa se igual a 1.
    * @property {string} pausa_motivo - Motivo da pausa
    * @property {number} max_chat_atende - Maximo de atendimento simultaneos de chats
    * @property {number} status - [0 - Desconhecido, 1 - Disponível, 2 - Ocupado, 3 - Ocupado, 4 - Invalido, 5 - Indisponível, 6 - Tocando, 7 - Desconhecido]
    * @property {number} status_date - Data e hora do status no formato Unix Timestamp
    * @property {number} ligacoes_efetuadas - Numero de ligações efetuadas
    * @property {number} ligacoes_recebidas - Numero de ligações recebidas
    * @property {number} ligacoes_naoatendidas - Numero de ligações não atendidas
    * @property {number} ligacoes_naocompletadas - Numero de ligações efetuadas mas não completadas
    */
    /**
     * Representação dos dados do agente dentro do Ami.
     * @param {agente} agente
     */
    constructor(agente) {
        Object.assign(this, agente)
    }
    static get AtributosMap() {
        return new Map([
            ["id", "id"],
            ["nome", "no"],
            ["ramal", "ra"],
            ["fila", "fl"],
            ["fila_padrao", "fp"],
            ["pausa", "pa"],
            ["pausa_motivo", "pm"],
            ["max_chat_atende", "ma"],
            ["status", "st"],
            ["status_date", "sd"],
            ["ligacoes_efetuadas", "le"],
            ["ligacoes_recebidas", "lr"],
            ["ligacoes_naoatendidas", "ln"],
            ["ligacoes_naocompletadas", "lp"]
        ]);
    }
    static getChavePeloValor(val) {
        return [...AgenteAmi.AtributosMap].find(([key, value]) => val === value)[0];
    }
    static get StatusMap() {
        return new Map([
            [0, "Desconhecido"],
            [1, "Disponível"],
            [2, "Ocupado"],
            [3, "Ocupado"],
            [4, "Invalido"],
            [5, "Indisponível"],
            [6, "Tocando"],
            [7, "Desconhecido"],
        ]);
    }
    /**
     * Converte os campos comprimidos de uma mensagem recebida via socket e retorna um objeto do tipo AgenteAmi.
     * @param {Object} msg 
     * @returns {Object} AgenteAmi
     */
    static Extrai(msg) {
        const ag = new AgenteAmi();
        for (let key of Object.keys(msg)) {
            const nk = AgenteAmi.getChavePeloValor(key);
            ag[nk] = msg[key];
        }
        return ag;
    }
    /**
     * Comprime os atributos do object retornando uma msg pronta para enviar via socket.
     */
    Comprime() {
        const msg = {};
        AgenteAmi.AtributosMap.forEach((value, key) => {
            if (this[key]) msg[value] = this[key];
        })
        return msg;
    }
}