module.exports = class LigacoesAmi {
    /**
    * @typedef fila
    * @type {object}
    * @property {number} callid - Identificador da ligacao
    * @property {number} inicio - Data e hora de inicio no formato Unix timestamp
    * @property {string} fila   - Nome da fila
    * @property {number} agente_id - ID do agente
    * @property {string} agente_nome - Nome do agente
    * @property {string} agente_ramal - Ramal do agente
    * @property {number} status - Status da ligação [1 - Chamando, 2 - Ativa, 3 - Finalizada, 4 - Nao_Atendida, 5 - Nao_Completada]
    * @property {number} sentido - Sentido da ligação [1 - Entrante, 2 - Sainte]
    * @property {string} numero - Numero de origem (callerID)
    * @property {string} nome - Nome associado ao número de origem (CalleridName)
    * @property {number} atendida - Data de atendimento no formato Unix timestamp
    * @property {number} fim - Data de termino da ligação no formato Unix timestamp
    * @property {number} posicao_inicial - Numero da posição em que a ligação entrou na fila
    * @property {number} posicao_final - Numero da posição que a ligação saiu da fila
    * @property {number} total - Total de ligações que tinha na fila quando a ligação entrou ou saiu da fila
    * @property {number} espera - Tempo em segundo que a ligação aguardou para ser atendida
    * @property {number} falando - Tempo em segundo que a ligação ficou em atendimento
    * @property {number} motivo_fim - Motivo pelo qual a ligação terminou [1 - Cliente Desligou, 2 - Agente Desligou, 3 - Transferida]
    */
    /**
     * Representação dos dados do agente dentro do Ami.
     * @param {fila} fila
     */
    constructor(fila) {
        Object.assign(this, fila)
    }
    static get AtributosMap() {
        return new Map([
            ["callid", "id"],
            ["inicio", "in"],
            ["fila", "fl"],
            ["agente_id", "ai"],
            ["agente_nome", "an"],
            ["agente_ramal", "ar"],
            ["status", "st"],
            ["sentido", "se"],
            ["numero", "nu"],
            ["nome", "no"],
            ["atendida", "at"],
            ["fim", "fi"],
            ["posicao_inicial", "pi"],
            ["posicao_final", "pf"],
            ["total", "to"],
            ["espera", "ep"],
            ["falando", "fa"],
            ["motivo_fim", "mf"]
        ]);
    }
    static get StatusMap() {
        return new Map([
            ['Chamando', 1],
            ['Ativa', 2],
            ['Finalizada', 3],
            ['Nao_Atendida', 4],
            ['Nao_Completada', 5]
        ]);
    }
    static get SentidoMap() {
        return new Map([
            ['Entrante', 1],
            ['Sainte', 2]
        ]);
    }
    static get MotivoFimMap() {
        return new Map([
            ['caller', 1],
            ['agent', 2],
            ['transfer', 3]
        ]);
    }
    static getChavePeloValor(val) {
        return [...LigacoesAmi.AtributosMap].find(([key, value]) => val === value)[0];
    }
    /**
     * Converte os campos comprimidos de uma mensagem recebida via socket e retorna um objeto do tipo LigacoesAmi.
     * @param {Object} msg 
     * @returns {Object} LigacoesAmi
     */
    static Extrai(msg) {
        const ag = new LigacoesAmi();
        for (let key of Object.keys(msg)) {
            const nk = LigacoesAmi.getChavePeloValor(key);
            ag[nk] = msg[key];
        }
        return ag;
    }
    /**
     * Comprime os atributos do object retornando uma msg pronta para enviar via socket.
     */
    Comprime() {
        const msg = {};
        LigacoesAmi.AtributosMap.forEach((value, key) => {
            if (this[key]) msg[value] = this[key];
        })
        return msg;
    }
}