"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const knorm_1 = require("@knorm/knorm");
class KnormPaginate {
    constructor({ name = 'paginate', page = 1, perPage = 10 } = {}) {
        this.name = name;
        this.page = parseInt(page);
        this.perPage = parseInt(perPage);
    }
    updateModel(knorm) {
        const { Model } = knorm;
        class PaginateModel extends Model {
            static count(options) {
                return __awaiter(this, void 0, void 0, function* () {
                    return this.query.count(options);
                });
            }
        }
        knorm.updateModel(PaginateModel);
    }
    updateQuery(knorm) {
        const { Query } = knorm;
        const defaultPage = this.page;
        const defaultPerPage = this.perPage;
        class PaginateQuery extends Query {
            constructor(model) {
                super(model);
                this.setOption('withPaginationData', true);
            }
            withPaginationData(withPaginationData = true) {
                return this.setOption('withPaginationData', !!withPaginationData);
            }
            page(page) {
                return this.setOption('page', page);
            }
            perPage(perPage) {
                return this.setOption('perPage', parseInt(perPage));
            }
            fetch(options) {
                const _super = Object.create(null, {
                    fetch: { get: () => super.fetch }
                });
                return __awaiter(this, void 0, void 0, function* () {
                    this.setOptions(options);
                    let { page, perPage } = this.options;
                    const { withPaginationData } = this.options;
                    if (page === undefined && perPage === undefined) {
                        return _super.fetch.call(this);
                    }
                    if (page === undefined) {
                        page = defaultPage;
                    }
                    if (perPage === undefined) {
                        perPage = defaultPerPage;
                    }
                    let total;
                    if (withPaginationData || page === 'last') {
                        const clone = this.clone().unsetOptions([
                            'groupBy',
                            'orderBy',
                            'limit',
                            'offset',
                        ]);
                        if (!clone.hasOption('distinct')) {
                            clone.unsetOption('fields');
                        }
                        total = yield clone.count();
                        if (page === 'last') {
                            page = Math.ceil(total / perPage);
                        }
                    }
                    page = page === 'first' ? 1 : parseInt(page);
                    const rows = yield _super.fetch.call(this, {
                        limit: perPage,
                        offset: perPage * (page - 1),
                    });
                    return withPaginationData ? { total, page, perPage, rows } : rows;
                });
            }
            prepareSql(sql, options) {
                const _super = Object.create(null, {
                    prepareSql: { get: () => super.prepareSql }
                });
                return __awaiter(this, void 0, void 0, function* () {
                    if (this.config.forCount) {
                        const joins = this.getOption('joins');
                        if (joins) {
                            joins.forEach((join) => {
                                // disable fields on joins if none have been set
                                if (!join.hasOption('fields')) {
                                    join.fields(false);
                                }
                                // enable recursively disabling joined fields
                                join.config.forCount = true;
                            });
                        }
                    }
                    return _super.prepareSql.call(this, sql, options);
                });
            }
            prepareCountFields() {
                let fields = [];
                if (this.options.fields) {
                    fields = fields.concat(Object.values(this.options.fields));
                    this.unsetOption('fields');
                }
                fields = fields.length
                    ? fields.map(this.getColumn, this).join(', ')
                    : '*';
                if (this.options.distinct) {
                    fields = `DISTINCT ${fields}`;
                    this.unsetOption('distinct');
                }
                return this.fields({ count: `COUNT(${fields})` });
            }
            count(options) {
                const _super = Object.create(null, {
                    fetch: { get: () => super.fetch }
                });
                return __awaiter(this, void 0, void 0, function* () {
                    this.setOptions(options);
                    this.prepareCountFields();
                    this.config.forCount = true;
                    const rows = yield _super.fetch.call(this);
                    const count = parseInt(rows[0].count);
                    if (!count && this.options.require) {
                        throw new this.constructor.NoRowsCountedError({ query: this });
                    }
                    return count;
                });
            }
        }
        PaginateQuery.NoRowsCountedError = class NoRowsCountedError extends Query.NoRowsError {
        };
        knorm.updateQuery(PaginateQuery);
    }
    init(knorm) {
        if (!knorm) {
            throw new this.constructor.KnormPaginateError('no Knorm instance provided');
        }
        if (!(knorm instanceof knorm_1.Knorm)) {
            throw new this.constructor.KnormPaginateError('invalid Knorm instance provided');
        }
        this.updateModel(knorm);
        this.updateQuery(knorm);
    }
}
exports.KnormPaginate = KnormPaginate;
KnormPaginate.KnormPaginateError = class KnormPaginateError extends knorm_1.Knorm.KnormError {
};
