const moment = require('moment-timezone');
const config = require('../config.json');
const db = require('../util/db');
const log = require('simple-node-logger').createSimpleLogger(config.log_ops);
log.setLevel(config.log ? config.log : 'error');

// Configura a timezone
moment.tz.setDefault(config.servidor.timezone);

class Horarios extends db.orm.Model {}

Horarios.table = 'horarios';
Horarios.fields = {
    id: {  type: 'integer', primary: true, updated: false, methods: true },
    nome: 'string',
    horaIni: {column: 'hora_ini', type: 'dateTime'},
    horaFim: {column: 'hora_fim', type: 'dateTime'},
    dias: 'object',
    messes: 'object',
    diasSemana: {column: 'dias_semana', type: 'object'},
    descricao: 'text',
    tipo: 'string'
};

//#region Funções

/***
 * @description Esta função verifica se os turnos estão válidos
 * Recebe um array de id's referente a tabela de horários
 *
 * Toda validação de data está dentro da função
 *
 * @author Leandro
 * @example Horarios.verificarTurnos([5])
 *
 * @param {Array<Number>} idTurnos Array contendo os turnos a servem validados
 *
 * @returns {Promise<Boolean>} True se estiver dentro de qualquer turno
 ***/
Horarios.verificarTurnos = function(idTurnos) {
    return new Promise((resolve, reject) => {
        // Pega a data atual
        let date = moment();
        let horario = date.format("HH:mm");
        let dia = parseInt(date.format("DD"));
        let diaSemana = date.isoWeekday();
        if (diaSemana >= 7) diaSemana = 0; // O Dia 7(Domingo) deve ser o primeiro dia da semana sendo o número 0
        let mes = parseInt(date.format("MM"));

        // Define a query para buscar turnos válidos
        const sql = /*sql*/`SELECT count(*) AS total FROM public.horarios WHERE tipo='T' AND id = ANY(ARRAY[${idTurnos}]) AND
                                    (hora_ini IS null OR hora_ini <= '${horario}') AND
                                    (hora_fim IS null OR hora_fim >= '${horario}') AND
                                    (dias IS null OR  dias = '{}' OR ${dia} = ANY(dias)) AND
                                    (messes IS null OR messes = '{}' OR ${mes} = ANY(messes)) AND
                                    (dias_semana IS null OR dias_semana = '{}' OR ${diaSemana} = ANY(dias_semana))`;

        // Executa a query
        db.query(sql)
            .then(queryResult => {
                // Verifica se o retorno da query é válido
                if (!queryResult || !queryResult.rows || queryResult.rows.length <= 0 || !queryResult.rows.find(x => x) || queryResult.rows.find(x => x).total <= 0) {
                    resolve(false);
                } else {
                    resolve(true);
                }
            })
            .catch(error => {
                console.trace(error);
                log.error("[DISCADOR-WHATSAPP] Erro ao verificar turnos da campanha: " + error);
                reject(error);
            });
    });
};

//#endregion


module.exports = Horarios;