/**
 * Este método limpa todos os caracteres que não são dígitos de um texto
 * 
 * @param {Any} input Parâmetro de entrada, podendo ser texto ou numérico
 * @returns {String} O parâmetro {@link input} após o processamento
 * 
 * @since 29/05/2020
 * @author Leandro Quevedo
 **/
exports.limparTextoNumerico = function(input) {
    try {        
        if (!input) input = "";
        if (typeof input !== "string") return input;
    
        const regex = /[^0-9]+/g;
    
        let after = input.replace(regex, "");
        if (input !== after) console.warn(`[TEXTO-UTIL] limparTextoNumerico -> Antes '${input}' Depois '${after}'`)
        return after;
    } catch (e) {
        console.error(`[TEXTO-UTIL] limparTextoNumerico -> ` + e);
        console.trace(e);
        return input;
    }
};

/**
 * Este método remove todos os acentos e caracteres especiais de um texto
 * 
 * @param {Any} input Parâmetro de entrada, podendo ser texto ou numérico
 * @returns {String} O parâmetro {@link input} após o processamento
 * 
 * @since 29/05/2020
 * @author Leandro Quevedo
 **/
exports.limparTexto = function(input) {
    try {
        if (!input) input = "";
        if (typeof input !== "string") return input;

        const accents = 'ÀÁÂÃÄÅàáâãäåßÒÓÔÕÕÖØòóôõöøÈÉÊËèéêëðÇçÐÌÍÎÏìíîïÙÚÛÜùúûüÑñŠšŸÿýŽž';
        const accentsOut = "AAAAAAaaaaaaBOOOOOOOooooooEEEEeeeeeCcDIIIIiiiiUUUUuuuuNnSsYyyZz";    

        let chars = input.trim().split("");

        for (let i = 0; i < chars.length; i++) {
            let letter = chars[i];
            let accentsIndex = accents.indexOf(letter);

            if (accentsIndex !== -1) {
                chars[i] = accentsOut[accentsIndex];
            }
        }

        let after = chars.join(""); 
        
        if (input !== after) console.warn(`[TEXTO-UTIL] limparTexto -> Antes '${input}' Depois '${after}'`)

        return after;
    } catch (e) {
        console.error(`[TEXTO-UTIL] limparTexto -> ` + e);
        console.trace(e);
        return input;
    }
};