const log = require("node-live-log")("util/QueueAgenteLogin");
const QueueEvents = require('xcontact-lib/models/QueueEvents');

const queue = require('queue');
const EventEmitter = require("events")
const myEmitter = new EventEmitter()

const q = queue({
    autostart: true,
    concurrency: 1,
    timeout: 10000
});

function makeJob (loginData) {
    const job = async function () {
        log.debug(`[makeJob] Iniciando Job para dados:`, loginData);
        try {
            return await new QueueEvents({
                event: 'Login',
                event_start: loginData.event_start,
                envent_group: 1,
                agent: loginData.agent,
                queue: loginData.queue,
            }).save()
        } catch (e) {
            log.error(`[makeJob] Erro:`, e);
            job.reject = e;
        }
    };

    job.cancel = function () {
        if (job.reject) {
            log.warn(`[makeJob] Cancelado`);
            job.reject({ canceled: true });
        }
    };
    return job;
}

function on(event, listener) {
    return myEmitter.on(event, listener)
}

function off(event, listener) {
    return myEmitter.off(event, listener)
}

q.on('success', function (result, job) {
    log.debug(`[q] Job finalizado com sucesso:`, result);
    myEmitter.emit('success', result)
});

q.on('timeout', function (next, job) {
    log.warn(`[q] Job timeout`);
    job.cancel();
    next();
});

function addJob (loginData) {
    const job = makeJob(loginData);
    q.push(job);
}

module.exports = {
    addJob,
    on,
    off
}
