const log = require("node-live-log")("util/QueueAgenteLogoff");
const QueueEvents = require('xcontact-lib/models/QueueEvents');

const queue = require('queue');
const EventEmitter = require("events")
const myEmitter = new EventEmitter()

const q = queue({
    autostart: true,
    concurrency: 1,
    timeout: 10000
});

function makeJob (logoffData) {
    const job = async function () {
        log.debug(`[makeJob] Iniciando Job para dados:`, logoffData);
        try {
            // fecha eventos de pausa do agente se houver
            await QueueEvents.query.require(false)
                .where(QueueEvents.where.gte('event_start', new Date(new Date().setDate(new Date().getDate() - 5))))
                .where('event', 'Pause')
                .where('event_end', null)
                .where('agent', logoffData.agent)
                .update({ event_end: logoffData.event_end })

            // fecha eventos de login do agente
            return await QueueEvents.query.require(false)
                .where(QueueEvents.where.gte('event_start', new Date(new Date().setDate(new Date().getDate() - 5))))
                .where('event', 'Login')
                .where('event_end', null)
                .where('agent', logoffData.agent)
                .where('queue', logoffData.queue)
                .update({ event_end: logoffData.event_end })
        } catch (e) {
            log.error(`[makeJob] Erro:`, e);
            job.reject = e;
        }
    };

    job.cancel = function () {
        if (job.reject) {
            log.warn(`[makeJob] Cancelado`);
            job.reject({ canceled: true });
        }
    };
    return job;
}

function on(event, listener) {
    return myEmitter.on(event, listener)
}

function off(event, listener) {
    return myEmitter.off(event, listener)
}

q.on('success', function (result, job) {
    log.debug(`[q] Job finalizado com sucesso:`, result);
    myEmitter.emit('success', result)
});

q.on('timeout', function (next, job) {
    log.warn(`[q] Job timeout`);
    job.cancel();
    next();
});

function addJob (loginData) {
    const job = makeJob(loginData);
    q.push(job);
}

module.exports = {
    addJob,
    on,
    off
}
