let logLevel = process.env.XCLOG_LEVEL || 'error'
let logLevels = ['error', 'warn', 'info', 'debug']

// Formata consolo.log para utilizar como logs
function logFormatter(msg, type, ...args) {
    if (msg instanceof Error) {
        msg = msg.stack
    }
    if (typeof msg === 'object') {
        msg = JSON.stringify(msg)
    }
    for (const arg of args) {
        if (arg instanceof Error) {
            msg += ` ${arg.stack}`
        }
        else if (typeof arg === 'object') {
            msg += ` ${JSON.stringify(arg)}`
        } else {
            msg += ` ${arg}`
        }
    }
    const logType = type ? `- [${type.toUpperCase()}]` : '-'
    return `[${new Date().toLocaleString()}] ${logType} ${msg}`
}

const log = (msg, ...args) => {
    if (['debug', 'info'].includes(logLevel)) {
        console.log(logFormatter(msg, null, ...args))
    }
}

log.info = (msg, ...args) => {
    if (['debug', 'info'].includes(logLevel)) {
        console.info(logFormatter(msg, 'info', ...args))
    }
}

log.error = (msg, ...args) => {
    console.error(logFormatter(msg, 'error', ...args))
}

log.warn = (msg, ...args) => {
    if (['warn', 'info', 'debug'].includes(logLevel)) {
        console.warn(logFormatter(msg, 'warn', ...args))
    }
}

log.debug = (msg, ...args) => {
    if (logLevel === 'debug') {
        console.debug(logFormatter(msg, 'debug', ...args))
    }
}

module.exports = log
module.exports.logLevel = logLevel